@@ -77,6 +77,7 @@
 #include "ap_Strings.h"
 #include "ap_Dialog_Latex.h"
 #include "ut_mbtowc.h"
+#include "ap_Menu_Id.h"
 
 #include "ut_sleep.h"
 #include <sys/types.h>  
@@ -118,7 +119,11 @@ static const char* AbiMathView_MenuLabelFileInsert = NULL;
 static const char* AbiMathView_MenuTooltipFileInsert = NULL;
 static const char* AbiMathView_MenuLabelLatexInsert = NULL;
 static const char* AbiMathView_MenuTooltipLatexInsert = NULL;
-
+static const char* AbiMathView_MenuEndEquation= "EndEquation";
+static XAP_Menu_Id newEquationID;
+static XAP_Menu_Id FromFileID;
+static XAP_Menu_Id FromLatexID;
+static XAP_Menu_Id endEquationID;
 static void AbiMathView_addToMenus()
 {
     // First we need to get a pointer to the application itself.
@@ -170,7 +175,7 @@ static void AbiMathView_addToMenus()
 
 // Put it after Insert Picture in the Main menu
 
-    XAP_Menu_Id newEquationID= pFact->addNewMenuAfter("Main",NULL,"&Endnote",EV_MLF_BeginSubMenu);
+    newEquationID= pFact->addNewMenuAfter("Main",NULL,AP_MENU_ID_INSERT_ENDNOTE,EV_MLF_BeginSubMenu);
    UT_DEBUGMSG(("newEquationID %d \n",newEquationID));
 
 
@@ -195,7 +200,7 @@ static void AbiMathView_addToMenus()
 
     pActionSet->addAction(myEquationAction);
 
-    XAP_Menu_Id FromFileID= pFact->addNewMenuAfter("Main",NULL,AbiMathView_MenuLabelEquation,EV_MLF_Normal);
+    FromFileID= pFact->addNewMenuAfter("Main",NULL,newEquationID,EV_MLF_Normal);
     UT_DEBUGMSG(("FromFile ID %d \n",FromFileID));
 
     pFact->addNewLabel(NULL,FromFileID,AbiMathView_MenuLabelFileInsert, AbiMathView_MenuTooltipFileInsert);
@@ -220,7 +225,7 @@ static void AbiMathView_addToMenus()
 
     pActionSet->addAction(myFileAction);
 
-   XAP_Menu_Id FromLatexID= pFact->addNewMenuAfter("Main",NULL,AbiMathView_MenuLabelFileInsert,EV_MLF_Normal);
+   FromLatexID= pFact->addNewMenuAfter("Main",NULL,FromFileID,EV_MLF_Normal);
    UT_DEBUGMSG(("Latex ID %d \n",FromLatexID));
  
     pFact->addNewLabel(NULL,FromLatexID,AbiMathView_MenuLabelLatexInsert, AbiMathView_MenuTooltipLatexInsert);
@@ -241,9 +246,9 @@ static void AbiMathView_addToMenus()
 
     pActionSet->addAction(myLatexAction);
 
-   XAP_Menu_Id endEquationID= pFact->addNewMenuAfter("Main",NULL,AbiMathView_MenuLabelLatexInsert,EV_MLF_EndSubMenu);
+   endEquationID= pFact->addNewMenuAfter("Main",NULL,AbiMathView_MenuLabelLatexInsert,EV_MLF_EndSubMenu);
    UT_DEBUGMSG(("End Equation ID %d \n",endEquationID));
-    pFact->addNewLabel(NULL,endEquationID,NULL,NULL);
+    pFact->addNewLabel(NULL,endEquationID,AbiMathView_MenuEndEquation,NULL);
 
  
  // Create the Action that will be called.
@@ -275,13 +280,17 @@ AbiMathView_removeFromMenus ()
 	EV_EditMethod * pEM = ev_EditMethod_lookup ( "AbiMathView_FileInsert" ) ;
 	pEMC->removeEditMethod ( pEM ) ;
 	DELETEP( pEM ) ;
+	pEM = ev_EditMethod_lookup ( "AbiMathView_LatexInsert" ) ;
+	pEMC->removeEditMethod ( pEM ) ;
+	DELETEP( pEM ) ;
 
 	// now remove crap from the menus
 	XAP_Menu_Factory * pFact = pApp->getMenuFactory();
 
-	pFact->removeMenuItem("Main",NULL,AbiMathView_MenuLabelEquation);
-	pFact->removeMenuItem("Main",NULL,AbiMathView_MenuLabelFileInsert);
-	pFact->removeMenuItem("Main",NULL,AbiMathView_MenuLabelLatexInsert);
+	pFact->removeMenuItem("Main",NULL,newEquationID);
+	pFact->removeMenuItem("Main",NULL,FromFileID);
+	pFact->removeMenuItem("Main",NULL,FromLatexID);
+	pFact->removeMenuItem("Main",NULL, endEquationID);
 
 	pApp->rebuildMenus();
 }
@@ -342,7 +351,7 @@ static bool s_AskForMathMLPathname(XAP_Frame * pFrame,
 	// raise the file-open dialog for inserting a MathML equation.
 	// return a_OK or a_CANCEL depending on which button
 	// the user hits.
-	// return a pointer a UT_strdup()'d string containing the
+	// return a pointer a g_strdup()'d string containing the
 	// pathname the user entered -- ownership of this goes
 	// to the caller (so free it when you're done with it).
 
@@ -379,7 +388,7 @@ static bool s_AskForMathMLPathname(XAP_Frame * pFrame,
 		const char * szResultPathname = pDialog->getPathname();
 		UT_DEBUGMSG(("MATHML Path Name selected = %s \n",szResultPathname));
 		if (szResultPathname && *szResultPathname)
-			UT_cloneString(*ppPathname,szResultPathname);
+			*ppPathname = g_strdup(szResultPathname);
 
 		UT_sint32 type = pDialog->getFileType();
 
@@ -451,7 +460,7 @@ AbiMathView_FileInsert(AV_View* v, EV_EditMethodCallData *d)
 
 	/* Create the data item */
 	const char* mimetypeMATHML = NULL;
-	mimetypeMATHML = UT_strdup("application/mathml+xml");
+	mimetypeMATHML = g_strdup("application/mathml+xml");
 	UT_uint32 uid = pDoc->getUID(UT_UniqueId::Image);
 	UT_UTF8String sUID;
 	UT_UTF8String_sprintf(sUID,"%d",uid);
@@ -546,28 +555,34 @@ void GR_MathManager::initialize(void)
 	// load the gtkmathview settings and operator dictionaries from the private user directory, ...
 	UT_UTF8String userConfDir(pApp->getUserPrivateDirectory());
 	UT_UTF8String userDictDir(pApp->getUserPrivateDirectory());
+	UT_UTF8String userCombiningDictDir(pApp->getUserPrivateDirectory());
 	UT_UTF8String userLocalDictDir(pApp->getUserPrivateDirectory());
 #if defined(WIN32)
 	userConfDir += "\\math\\gtkmathview.conf.xml";
 	userDictDir += "\\math\\dictionary.xml";
+	userCombiningDictDir += "\\math\\dictionary-combining.xml";
 	userLocalDictDir += "\\math\\dictionary-local.xml";
 #else
 	userConfDir += "/math/gtkmathview.conf.xml";
 	userDictDir += "/math/dictionary.xml";
+	userCombiningDictDir += "/math/dictionary-combining.xml";
 	userLocalDictDir += "/math/dictionary-local.xml";
 #endif
 
 	// ... or from the (common) AbiSuite directory
 	UT_UTF8String libConfDir(pApp->getAbiSuiteLibDir());
 	UT_UTF8String libDictDir(pApp->getAbiSuiteLibDir());
+	UT_UTF8String libCombiningDictDir(pApp->getAbiSuiteLibDir());
 	UT_UTF8String libLocalDictDir(pApp->getAbiSuiteLibDir());
 #if defined(WIN32)	 
 	libConfDir += "\\math\\gtkmathview.conf.xml";
 	libDictDir += "\\math\\dictionary.xml";
+	libCombiningDictDir += "\\math\\dictionary-combining.xml";
 	libLocalDictDir += "\\math\\dictionary-local.xml";
 #else
 	libConfDir += "/math/gtkmathview.conf.xml";
 	libDictDir += "/math/dictionary.xml";
+	libCombiningDictDir += "/math/dictionary-combining.xml";
 	libLocalDictDir += "/math/dictionary-local.xml";
 #endif
 
@@ -595,22 +610,31 @@ void GR_MathManager::initialize(void)
 	if (const char * resources = getenv("ABIWORD_COCOA_BUNDLED_RESOURCES"))
 	{
 		UT_UTF8String bundleDictDir(resources);
+		UT_UTF8String bundleCombiningDictDir(resources);
 		UT_UTF8String bundleLocalDictDir(resources);
+
 		bundleDictDir += "/gtkmathview/dictionary.xml";
+		bundleCombiningDictDir += "/gtkmathview/dictionary-combining.xml";
 		bundleLocalDictDir += "/gtkmathview/dictionary-local.xml";
+
 		configuration->add("dictionary/path", bundleDictDir.utf8_str());
+		configuration->add("dictionary/path", bundleCombiningDictDir.utf8_str());
 		configuration->add("dictionary/path", bundleLocalDictDir.utf8_str());
 	}
 #endif
 	configuration->add("dictionary/path", libDictDir.utf8_str());
+	configuration->add("dictionary/path", libCombiningDictDir.utf8_str());
 	configuration->add("dictionary/path", libLocalDictDir.utf8_str());
+
 	configuration->add("dictionary/path", userDictDir.utf8_str());
+	configuration->add("dictionary/path", userCombiningDictDir.utf8_str());
 	configuration->add("dictionary/path", userLocalDictDir.utf8_str());
 
 	SmartPtr<GR_Abi_MathGraphicDevice> mathGraphicDevice 
 			= GR_Abi_MathGraphicDevice::create(logger, configuration, getGraphics());
 	m_pMathGraphicDevice = mathGraphicDevice;
 	m_pAbiContext = new GR_Abi_RenderingContext(getGraphics());
+	UT_DEBUGMSG(("MAthView New rendering context %x Graphics %x \n",m_pAbiContext,getGraphics()));
 	m_pOperatorDictionary = initOperatorDictionary<libxml2_MathView>(logger, configuration);
 }
 
@@ -725,12 +749,16 @@ bool GR_MathManager::createPNGSnapshot(AD_Document * pDoc, UT_Rect & rec,
 
 	GR_Painter painter(getGraphics());
 	GR_Image * pImage = painter.genImageFromRectangle(rec);
+	if(pImage == NULL)
+	{
+		return false;
+	}
 	UT_ByteBuf * pBuf = NULL;
 	pImage->convertToBuffer(&pBuf);
 	UT_UTF8String sID = "snapshot-png-";
 	sID += szDataID;
 
-	mimetypePNG = UT_strdup("image/png");
+	mimetypePNG = g_strdup("image/png");
 	UT_DEBUGMSG(("Making data-item of name %s \n",sID.utf8_str()));
 	pDoc->createDataItem(sID.utf8_str(),false,reinterpret_cast< const UT_ByteBuf *>(pBuf),mimetypePNG,NULL);
 
@@ -810,7 +838,7 @@ void GR_MathManager::loadEmbedData(UT_sint32 uid)
 		bFoundDataID = m_pDoc->getDataItemDataByName(pszDataID, 
 						const_cast<const UT_ByteBuf **>(&pByteBuf),
 						NULL, NULL);
-
+		UT_return_if_fail(pByteBuf && bFoundDataID);
 		UT_UCS4_mbtowc myWC;
 		sMathML.appendBuf( *pByteBuf, myWC);
 	}
@@ -833,6 +861,7 @@ UT_sint32 GR_MathManager::getAscent(UT_sint32 uid)
 {
 	SmartPtr<libxml2_MathView>  pMathView = m_vecMathView.getNthItem(uid);
 	BoundingBox box = pMathView->getBoundingBox();
+	UT_DEBUGMSG(("GetAscent New rendering context %x Graphics %x Device resolution %d \n",m_pAbiContext,getGraphics(),getGraphics()->getDeviceResolution()));
 	return m_pAbiContext->toAbiLayoutUnits(box.height);
 }
 
