@@ -27,10 +27,11 @@
 #include "ODe_Style_MasterPage.h"
 #include "ODe_Style_PageLayout.h"
 #include "ODe_Style_Style.h"
+#include "ut_misc.h"
 
 // External includes
 #include <gsf/gsf-outfile.h>
-
+#include <gsf/gsf-output-memory.h>
 
 /**
  * Constructor
@@ -56,7 +57,7 @@ ODe_DocumentData::~ODe_DocumentData() {
     DELETEP(pMasterPageVector);
     
     if (m_pOfficeTextTemp != NULL) {
-        fclose(m_pOfficeTextTemp);
+        ODe_gsf_output_close(m_pOfficeTextTemp);
     }
 }
 
@@ -93,7 +94,7 @@ bool ODe_DocumentData::doPreListeningWork(PD_Document* pAbiDoc) {
     m_masterStyles.insert("Standard", pMPStyle);
     
     
-    m_pOfficeTextTemp = tmpfile();
+    m_pOfficeTextTemp = gsf_output_memory_new();
     if (m_pOfficeTextTemp == NULL) {
         return false;
     }
@@ -206,7 +207,7 @@ bool ODe_DocumentData::writeStylesXML(GsfOutfile* pOdt) const {
     " xmlns:dom=\"http://www.w3.org/2001/xml-events\""
     " office:version=\"1.0\">\n"};
     
-    ODe_writeToStream(pStylesStream, preamble, NrElements(preamble));
+    ODe_writeToStream(pStylesStream, preamble, G_N_ELEMENTS(preamble));
     
     m_stylesXMLFontDecls.write(pStylesStream);
     
@@ -240,9 +241,6 @@ bool ODe_DocumentData::writeStylesXML(GsfOutfile* pOdt) const {
  */
 bool ODe_DocumentData::writeContentXML(GsfOutfile* pOdt) {
     GsfOutput* pContentStream;
-    size_t bytesRead;
-    UT_Byte byteBuffer[1000];
-    int intVal;
     
     pContentStream = gsf_outfile_new_child (pOdt, "content.xml", FALSE);
     
@@ -275,7 +273,7 @@ bool ODe_DocumentData::writeContentXML(GsfOutfile* pOdt) {
     " xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\""
     " office:version=\"1.0\">\n"};
     
-    ODe_writeToStream(pContentStream, preamble, NrElements(preamble));
+    ODe_writeToStream(pContentStream, preamble, G_N_ELEMENTS(preamble));
     
     m_contentXMLFontDecls.write(pContentStream);
     
@@ -283,18 +281,11 @@ bool ODe_DocumentData::writeContentXML(GsfOutfile* pOdt) {
     
     ODe_writeUTF8String(pContentStream, " <office:body>\n"
                                         "  <office:text>\n");
-                                        
-    rewind(m_pOfficeTextTemp);
+       
+    ODe_gsf_output_write(pContentStream, gsf_output_size (m_pOfficeTextTemp),
+			 gsf_output_memory_get_bytes (GSF_OUTPUT_MEMORY (m_pOfficeTextTemp)));
     
-    while (feof(m_pOfficeTextTemp) == 0) {
-        bytesRead = fread((void *)byteBuffer, 1, 1000, m_pOfficeTextTemp);
-        
-        ODe_gsf_output_write(pContentStream, bytesRead,
-                    (guint8 const *)byteBuffer);
-    }
-    
-    intVal = fclose(m_pOfficeTextTemp);
-    UT_ASSERT(intVal == 0);
+    ODe_gsf_output_close (m_pOfficeTextTemp);
     m_pOfficeTextTemp = NULL;
     
     ODe_writeUTF8String(pContentStream,
