@@ -33,7 +33,7 @@
 
 // AbiWord includes
 #include <pp_AttrProp.h>
-
+#include <gsf/gsf-output-memory.h>
 
 /**
  * Constructor
@@ -89,7 +89,7 @@ void ODe_Main_Listener::openSection(const PP_AttrProp* pAP,
 
             pPageLayout = m_rDocumentData.m_stylesAutoStyles.getPageLayout("Standard");
             pMPStyle = m_rDocumentData.m_masterStyles.pick("Standard");
-            
+            UT_DEBUGMSG(("Got PageLayout %x AutoStyles %x \n",pPageLayout,&m_rDocumentData.m_stylesAutoStyles));
             m_isFirstSection = false;
 
         } else {
@@ -114,8 +114,16 @@ void ODe_Main_Listener::openSection(const PP_AttrProp* pAP,
         
         pMPStyle->fetchAttributesFromAbiSection(pAP);
         pPageLayout->fetchAttributesFromAbiSection(pAP);
+	//
+	// OK Set up a "standard" default set of properties
+	//
+	ODe_Style_PageLayout* pStandard = new ODe_Style_PageLayout();
+	UT_UTF8String sName = "Standard";
+	pStandard->setName(sName);
+        m_rDocumentData.m_contentAutoStyles.addPageLayout(pStandard);
+	pStandard->fetchAttributesFromAbiSection(pAP);
     } else {
-        // Without this, '!UT_strcmp(pId, pValue)' fails in _isHeaderFooterSection()
+        // Without this, '!strcmp(pId, pValue)' fails in _isHeaderFooterSection()
         // below, which ultimately leads to a crash due to an uninitialized
         // FILE* being passed to fwrite() (see bug 9798 for a sample).
         pMPStyle = m_rDocumentData.m_masterStyles.pick("Standard");
@@ -131,7 +139,10 @@ void ODe_Main_Listener::openSection(const PP_AttrProp* pAP,
         
         pSectionStyle->fetchAttributesFromAbiSection(pAP);
         m_rDocumentData.m_contentAutoStyles.storeSectionStyle(pSectionStyle);
-        
+
+        ODe_Style_PageLayout* pPageLayout = m_rDocumentData.m_contentAutoStyles.addPageLayout();
+	pPageLayout->fetchAttributesFromAbiSection(pAP);
+       
         UT_UTF8String output;
         
         UT_UTF8String_sprintf(output,
@@ -165,7 +176,8 @@ void ODe_Main_Listener::openSection(const PP_AttrProp* pAP,
 /**
  * Override of ODe_AbiDocListenerImpl::closeSection
  */
-void ODe_Main_Listener::closeSection(ODe_ListenerAction& rAction) {
+void ODe_Main_Listener::closeSection(ODe_ListenerAction& /*rAction*/) 
+{
 	if (m_openedODSection) {
 
 		ODe_writeToFile(m_rDocumentData.m_pOfficeTextTemp, "   </text:section>\n");
@@ -181,12 +193,12 @@ void ODe_Main_Listener::closeSection(ODe_ListenerAction& rAction) {
  * 
  */
 bool ODe_Main_Listener::_isHeaderFooterSection(const PP_AttrProp* pAP) const {
-    const XML_Char* pValue;
+    const gchar* pValue;
     bool ok;
     
     ok = pAP->getAttribute("type", pValue);
     if (ok && pValue != NULL) {
-        if (!UT_strcmp(pValue, "header") || !UT_strcmp(pValue, "footer")) {
+        if (!strcmp(pValue, "header") || !strcmp(pValue, "footer")) {
             return true;
         }
     }
@@ -201,13 +213,13 @@ bool ODe_Main_Listener::_isHeaderFooterSection(const PP_AttrProp* pAP) const {
 void ODe_Main_Listener::_openHeaderFooterSection(
                                                   const PP_AttrProp* pAP,
                                                   ODe_ListenerAction& rAction) {
-    const XML_Char* pValue;
-    const XML_Char* pId;
+    const gchar* pValue;
+    const gchar* pId = NULL;
     bool ok;
     UT_GenericVector<ODe_Style_MasterPage*>* pMasterPageVector;
     UT_uint32 count, i;
     const ODe_Style_MasterPage* pMPageStyle;
-    FILE* pTextOutput;
+    GsfOutput* pTextOutput = NULL;
     
     pMasterPageVector = m_rDocumentData.m_masterStyles.enumerate();
     count = pMasterPageVector->getItemCount();
@@ -227,12 +239,12 @@ void ODe_Main_Listener::_openHeaderFooterSection(
     }
     
     ok = false;
-    if (!UT_strcmp("header", pValue)) {
+    if (!strcmp("header", pValue)) {
         
         for (i=0; i<count && !ok; i++) {
             pMPageStyle = (*pMasterPageVector)[i];
             pValue = pMPageStyle->getAbiHeaderId().utf8_str();
-            if (!UT_strcmp(pId, pValue)) {
+            if (!strcmp(pId, pValue)) {
                 ok = true; // found it. get out of this "for" loop
                 pTextOutput =  pMPageStyle->getHeaderContentTempFile();
             }
@@ -244,7 +256,7 @@ void ODe_Main_Listener::_openHeaderFooterSection(
         for (i=0; i<count && !ok; i++) {
             pMPageStyle = (*pMasterPageVector)[i];
             pValue = pMPageStyle->getAbiFooterId().utf8_str();
-            if (!UT_strcmp(pId, pValue)) {
+            if (!strcmp(pId, pValue)) {
                 ok = true; // found it. get out of this "for" loop
                 pTextOutput = pMPageStyle->getFooterContentTempFile();
             }
@@ -253,7 +265,7 @@ void ODe_Main_Listener::_openHeaderFooterSection(
 
     if (!ok) {
         UT_ASSERT(UT_SHOULD_NOT_HAPPEN); // We should have found this header/footer.
-        pTextOutput = tmpfile();
+        pTextOutput = gsf_output_memory_new ();
     }
 
     m_openedODSection = false;
