@@ -1,3 +1,5 @@
+/* -*- mode: C++; tab-width: 4; c-basic-offset: 4; -*- */
+
 /* AbiWord
  * Copyright (C) 2001 AbiSource, Inc.
  * Copyright (C) 2001 Petr Tomasek <tomasek@etf.cuni.cz>
@@ -58,10 +60,6 @@ int abi_plugin_register (XAP_ModuleInfo * mi)
 	{
 		m_sniffer = new IE_Imp_T602_Sniffer ();
 	}
-	else
-	{
-		m_sniffer->ref();
-	}
 
 	UT_ASSERT (m_sniffer);
 
@@ -89,10 +87,8 @@ int abi_plugin_unregister (XAP_ModuleInfo * mi)
 	UT_ASSERT (m_sniffer);
 
 	IE_Imp::unregisterImporter (m_sniffer);
-	if (!m_sniffer->unref())
-	{
-		m_sniffer = 0;
-	}
+	delete m_sniffer;
+	m_sniffer = 0;
 
 	return 1;
 }
@@ -112,6 +108,19 @@ IE_Imp_T602_Sniffer::IE_Imp_T602_Sniffer () :
   // 
 }
 
+// supported suffixes
+static IE_SuffixConfidence IE_Imp_T602_Sniffer__SuffixConfidence[] = {
+	{ "602", 	UT_CONFIDENCE_PERFECT 	},
+	{ "t602", 	UT_CONFIDENCE_PERFECT 	},
+	{ "txt", 	UT_CONFIDENCE_POOR 		},
+	{ "", 	UT_CONFIDENCE_ZILCH 	}
+};
+
+const IE_SuffixConfidence * IE_Imp_T602_Sniffer::getSuffixConfidence ()
+{
+	return IE_Imp_T602_Sniffer__SuffixConfidence;
+}
+
 UT_Confidence_t IE_Imp_T602_Sniffer::recognizeContents (const char * szBuf, 
 						     UT_uint32 iNumbytes)
 {
@@ -120,18 +129,6 @@ UT_Confidence_t IE_Imp_T602_Sniffer::recognizeContents (const char * szBuf,
   return UT_CONFIDENCE_ZILCH;   
 }
 
-UT_Confidence_t IE_Imp_T602_Sniffer::recognizeSuffix (const char * szSuffix)
-{
-  if ((!UT_stricmp(szSuffix,".602")) ||
-      (!UT_stricmp(szSuffix,".t602")))
-    return UT_CONFIDENCE_PERFECT;
-  
-  if (!UT_stricmp(szSuffix,".txt"))
-    return UT_CONFIDENCE_POOR;
-
-  return UT_CONFIDENCE_ZILCH;
-}
-
 UT_Error IE_Imp_T602_Sniffer::constructImporter (PD_Document * pDocument,
 													  IE_Imp ** ppie)
 {
@@ -306,7 +303,7 @@ UT_uint16 IE_Imp_T602::_conv(unsigned char c)
 bool IE_Imp_T602::_getbyte(unsigned char &c)
 {
 	UT_ASSERT(m_importFile);
-	return (fread (&c, 1, sizeof(c), m_importFile) > 0);
+	return (gsf_input_read (m_importFile, 1, &c) != NULL);
 }
 
 UT_Error IE_Imp_T602::_writeheader()
@@ -322,7 +319,7 @@ UT_Error IE_Imp_T602::_writeTP()
 {
   UT_DEBUGMSG(("T602: Append text properties\n"));
   UT_String buff;
-  const XML_Char* pps[3];
+  const gchar* pps[3];
   UT_String_sprintf(buff,"font-family: %s; font-size: %dpt; color:%s; font-weight: %s; "
 		    "font-style: %s; text-decoration: %s; text-position: %s",
 		    m_family.c_str(), m_size, m_color.c_str(),
@@ -344,7 +341,7 @@ UT_Error IE_Imp_T602::_writePP()
 {
   UT_DEBUGMSG(("T602: Append paragraph properties\n"));
   UT_String buff;
-  const XML_Char* pps[3];
+  const gchar* pps[3];
 
   // Don't put %1.1f here!! Locales could fuck it...
   UT_String_sprintf(buff,"line-height: %d.%d",
@@ -361,7 +358,7 @@ UT_DEBUGMSG(("T602: par-prop:\"%s\"]\n",buff.c_str()));
 UT_Error IE_Imp_T602::_writeSP()
 {
   UT_DEBUGMSG(("T602: Append section\n"));
-  const XML_Char* sps[7];
+  const gchar* sps[7];
   UT_String bf1, bf2, buff;
   int i=2;
 
@@ -382,14 +379,14 @@ UT_Error IE_Imp_T602::_writeSP()
   {
     sps[i]="header";
    UT_String_sprintf(bf1,"%d",m_header);
-   sps[i+1]=static_cast<const XML_Char *>(bf1.c_str());
+   sps[i+1]=static_cast<const gchar *>(bf1.c_str());
    i=2;
   }
   if (m_footer)
   {
    sps[i]="footer";
    UT_String_sprintf(bf2,"%d",m_footer);
-   sps[i+1]=static_cast<const XML_Char *>(bf2.c_str());
+   sps[i+1]=static_cast<const gchar *>(bf2.c_str());
    i+=2;
   }
   sps[i]=NULL;
@@ -401,8 +398,8 @@ UT_Error IE_Imp_T602::_writeSP()
 UT_Error IE_Imp_T602::_write_fh(UT_String & fh, UT_uint32 id, bool hea)
 {
   UT_DEBUGMSG(("T602: Append footer/header section\n"));
-  const XML_Char* fhps[5];
-  const XML_Char* fps[5];
+  const gchar* fhps[5];
+  const gchar* fps[5];
   UT_String bf1;
   UT_String buff;
   int i = 0;
@@ -410,7 +407,7 @@ UT_Error IE_Imp_T602::_write_fh(UT_String & fh, UT_uint32 id, bool hea)
 
   UT_String_sprintf(bf1,"%d",id);
   fhps[0]="id";
-  fhps[1]=static_cast<const XML_Char *>(bf1.c_str());
+  fhps[1]=static_cast<const gchar *>(bf1.c_str());
   fhps[2]="type";
   fhps[3]=(hea?"header":"footer");
   fhps[4]=NULL;
@@ -748,17 +745,12 @@ return UT_OK;
 }
 
 
-UT_Error IE_Imp_T602::importFile(const char * szFilename)
+UT_Error IE_Imp_T602::_loadFile(GsfInput * input)
 {
    UT_Error error;  
    unsigned char c;
    
-   m_importFile = fopen(szFilename, "rb");
-   if (!m_importFile)
-     {
-	UT_DEBUGMSG(("Could not open file %s\n",szFilename));
-	return UT_IE_FILENOTFOUND;
-     }
+   m_importFile = (GsfInput *)g_object_ref (G_OBJECT (input));
    
    error=UT_OK;
 	   
@@ -782,6 +774,6 @@ UT_Error IE_Imp_T602::importFile(const char * szFilename)
        X_CheckT602Error(_write_fh(m_hbuff, m_header, true));
      }
 
-   X_CheckT602Error(fclose(m_importFile))
+   g_object_unref(G_OBJECT(m_importFile));
    return error;
 }
