@@ -29,6 +29,7 @@
 #include "ut_debugmsg.h"
 #include "pt_Types.h"
 #include "ie_exp_WML.h"
+#include "ie_TOC.h"
 #include "pd_Document.h"
 #include "px_ChangeRecord.h"
 #include "px_CR_Object.h"
@@ -36,7 +37,8 @@
 #include "px_CR_Strux.h"
 #include "fd_Field.h"
 #include "xap_App.h"
-
+#include "pd_Style.h"
+#include "ap_Strings.h"
 #include "ut_path.h"
 #include "xap_EncodingManager.h"
 
@@ -67,7 +69,7 @@ IE_Exp_WML_Sniffer::IE_Exp_WML_Sniffer (const char * name) :
 
 bool IE_Exp_WML_Sniffer::recognizeSuffix(const char * szSuffix)
 {
-	return (!UT_stricmp(szSuffix,".wml"));
+	return (!g_ascii_strcasecmp(szSuffix,".wml"));
 }
 
 UT_Error IE_Exp_WML_Sniffer::constructExporter(PD_Document * pDocument,
@@ -108,21 +110,23 @@ UT_Error IE_Exp_WML::_writeDocument(void)
 
 s_WML_Listener::s_WML_Listener(PD_Document * pDocument,
 			       IE_Exp_WML * pie)
-	: mTableHelper(pDocument),
-	m_pDocument(pDocument),
+	: m_pDocument(pDocument),
 	m_pie(pie),
+	m_bInSection(false),
 	m_bInBlock(false),
 	m_bInSpan(false),
 	m_bInAnchor(false),
 	m_bInHyperlink(false),
 	m_bInCell(false),
 	m_bInRow(false),
-	m_bInSection(false),
 	m_bInTable(false),
 	m_bPendingClose(false),
 	m_bWasSpace(false),
 	m_iCards(1),
-	m_iTableDepth(0)
+	m_iTableDepth(0),
+	mTableHelper(pDocument),
+	m_toc(0),
+	m_heading_count(0)
 {
 	m_pie->write("<!DOCTYPE wml PUBLIC \"-//PHONE.COM//DTD WML 1.1//EN\"\n");
 	m_pie->write("\t\"http://www.openwave.com/dtd/wml11.dtd\" >\n");
@@ -132,6 +136,8 @@ s_WML_Listener::s_WML_Listener(PD_Document * pDocument,
 	m_pie->write("<!-- See http://www.abisource.com/ -->\n\n");
 	m_pie->write("<wml>\n");
 	_handleMetaData();
+
+	m_toc = new IE_TOCHelper(m_pDocument);
 }
 
 s_WML_Listener::~s_WML_Listener()
@@ -145,12 +151,13 @@ s_WML_Listener::~s_WML_Listener()
 	_handleDataItems();
 
 	UT_VECTOR_FREEALL(char*, m_utvDataIDs);
+	DELETEP(m_toc);
 }
 
 /*****************************************************************/
 /*****************************************************************/
 
-char *s_WML_Listener::_stripSuffix(const char* from, char delimiter)
+static char *_stripSuffix(const char* from, char delimiter)
 {
     char * fremove_s = static_cast<char *>(malloc(strlen(from)+1));
     strcpy(fremove_s, from);
@@ -165,6 +172,11 @@ char *s_WML_Listener::_stripSuffix(const char* from, char delimiter)
     return fremove_s;
 }
 
+static char *_stripSuffix(const UT_UTF8String &from, char delimiter)
+{
+  return _stripSuffix(from.utf8_str(), delimiter);
+}
+
 void s_WML_Listener::_closeAnchor()
 {
 	if(!m_bInAnchor)
@@ -291,13 +303,10 @@ void s_WML_Listener::_openSection(PT_AttrPropIndex api)
 {
 	const PP_AttrProp * pAP = NULL;
 	bool bHaveProp = m_pDocument->getAttrProp(api,&pAP);
-	const XML_Char * szValue = NULL;
+	const gchar * szValue = NULL;
 
 	if(pAP && bHaveProp && (pAP->getAttribute("strux-image-dataid", szValue)))
 	{
-		if(!UT_strcmp("",szValue))  // occurs with word documents
-			return;
-
 		_openSpan(api);
 		_handlePositionedImage(api);
 		_closeSpan();
@@ -338,6 +347,24 @@ void s_WML_Listener::_closeBlock(void)
 	return;
 }
 
+bool s_WML_Listener::_styleDescendsFrom(const char * style_name, const char * base_name)
+{
+	PD_Style * style = 0;
+	m_pDocument->getStyle (style_name, &style);
+	UT_sint32 iLoop = 0;
+
+	while (style && (iLoop < 10))
+	{
+		if (g_ascii_strcasecmp (base_name, style->getName ()) == 0)
+			return true;
+
+		style = style->getBasedOn ();
+		iLoop++;
+	}
+
+	return false;
+}
+
 void s_WML_Listener::_openParagraph(PT_AttrPropIndex api)
 {
 	xxx_UT_DEBUGMSG(("WML Export: OpenParagraph called!\n"));
@@ -356,14 +383,14 @@ void s_WML_Listener::_openParagraph(PT_AttrPropIndex api)
 	
 	if (bHaveProp && pAP)
 	{
-		const XML_Char * szValue;
+		const gchar * szValue;
 
 		m_pie->write("<p");
 		if (pAP->getProperty("text-align", szValue))
 		{
-		  if (!UT_strcmp(szValue, "center")) 
+		  if (!strcmp(szValue, "center")) 
 			m_pie->write(" align=\"center\"");
-		  else if (!UT_strcmp(szValue, "right"))
+		  else if (!strcmp(szValue, "right"))
 		        m_pie->write(" align=\"right\"");
 		  else //left or block-justify
 		        m_pie->write(" align=\"left\"");
@@ -377,9 +404,101 @@ void s_WML_Listener::_openParagraph(PT_AttrPropIndex api)
 	  m_pie->write("<p>");
 	}
 
+	const gchar * szStyle;
+	bool have_style  = pAP->getAttribute (PT_STYLE_ATTRIBUTE_NAME, szStyle);
+
+	if (have_style && szStyle)
+	{
+		if (m_toc)
+		{
+			if (_styleDescendsFrom(szStyle, "Heading 1") || _styleDescendsFrom(szStyle, "Heading 2") ||
+			    _styleDescendsFrom(szStyle, "Heading 3") || _styleDescendsFrom(szStyle, "Heading 4"))
+			{
+				UT_UTF8String tocAnchor(UT_UTF8String_sprintf("<anchor id=\"AbiTOC%d\"></anchor>", m_heading_count));
+				m_pie->write(tocAnchor.utf8_str());
+				m_heading_count++;
+			}
+		}
+	}
+
 	m_bInBlock = true;
 }
 
+void s_WML_Listener::_emitTOC (PT_AttrPropIndex api) {
+	int level1_depth = 0;
+	int level2_depth = 0;
+	int level3_depth = 0;
+	int level4_depth = 0;
+
+	const PP_AttrProp * pAP = 0;
+	bool bHaveProp = (api ? (m_pDocument->getAttrProp (api, &pAP)) : false);
+	bool bEmitHeading = true;
+	const gchar * szValue = 0;
+	UT_UTF8String tocHeadingUTF8;
+
+	_closeSpan();
+	_closeBlock();
+
+	if(bHaveProp && pAP && pAP->getProperty("toc-has-heading", szValue) && szValue) //check to see if the TOC heading is hidden
+	{
+		if(atoi(szValue) == 0)
+			bEmitHeading = false;
+	}
+
+	if(bEmitHeading)
+	{
+		if(bHaveProp && pAP && pAP->getProperty("toc-heading", szValue) && szValue) // user-defined TOC heading
+		{
+			tocHeadingUTF8 = szValue;
+		}
+		else
+		{
+			const XAP_StringSet * pSS = XAP_App::getApp()->getStringSet();
+			if(pSS)
+				pSS->getValueUTF8(AP_STRING_ID_TOC_TocHeading, tocHeadingUTF8);
+		}
+
+		//TODO: query the style properties to determine whether to center the TOC heading
+		m_pie->write("<p>");
+		m_pie->write(tocHeadingUTF8.escapeXML().utf8_str());
+		m_pie->write("</p>\n");
+	}
+
+	for (int i = 0; i < m_toc->getNumTOCEntries(); i++) {
+		int tocLevel = 0;		
+		
+		UT_UCS4String tocText(m_toc->getNthTOCEntry(i, &tocLevel).utf8_str());
+
+		m_pie->write("<p>");
+			
+		UT_UCS4String tocLevelText;
+		if(tocLevel == 1) {
+			level1_depth++;
+			level2_depth = level3_depth = level4_depth = 0;
+				
+			tocLevelText = UT_UTF8String_sprintf("[%d] ", level1_depth).ucs4_str();
+		} else if(tocLevel == 2) {
+			level2_depth++;
+			level3_depth = level4_depth = 0;
+			tocLevelText = UT_UTF8String_sprintf("[%d.%d] ", level1_depth, level2_depth).ucs4_str();
+		} else if(tocLevel == 3) {
+			level3_depth++;
+			level4_depth = 0;
+			tocLevelText = UT_UTF8String_sprintf("[%d.%d.%d] ", level1_depth, level2_depth, level3_depth).ucs4_str();
+		} else if(tocLevel == 4) {
+			level4_depth++;
+			tocLevelText = UT_UTF8String_sprintf("[%d.%d.%d.%d] ", level1_depth, level2_depth, level3_depth, level4_depth).ucs4_str();
+		}
+			
+		UT_UTF8String tocLink(UT_UTF8String_sprintf("<a href=\"#AbiTOC%d\">", i));
+		m_pie->write(tocLink.utf8_str(), tocLink.byteLength());
+		_outputDataUnchecked (tocLevelText.ucs4_str(), tocLevelText.length());
+		_outputDataUnchecked (tocText.ucs4_str(), tocText.length());
+		m_pie->write("</a>", 4);
+		m_pie->write("</p>");
+	}
+}
+
 /*****************************************************************/
 /*****************************************************************/
 
@@ -450,7 +569,7 @@ bool s_WML_Listener::populate(PL_StruxFmtHandle /*sfh*/,
 			}
 
 			default:
-				UT_ASSERT(UT_TODO);
+				UT_ASSERT_HARMLESS(UT_TODO);
 				return true;
 			}
 		}
@@ -459,7 +578,7 @@ bool s_WML_Listener::populate(PL_StruxFmtHandle /*sfh*/,
 		return true;
 		
 	default:
-		UT_ASSERT(UT_TODO);
+		UT_ASSERT_HARMLESS(UT_TODO);
 		return true;
 	}
 }
@@ -468,7 +587,7 @@ bool s_WML_Listener::populateStrux(PL_StruxDocHandle sdh,
 				      const PX_ChangeRecord * pcr,
 				      PL_StruxFmtHandle * psfh)
 {
-	UT_ASSERT(pcr->getType() == PX_ChangeRecord::PXT_InsertStrux);
+	UT_ASSERT_HARMLESS(pcr->getType() == PX_ChangeRecord::PXT_InsertStrux);
 	const PX_ChangeRecord_Strux * pcrx = static_cast<const PX_ChangeRecord_Strux *> (pcr);
 	*psfh = 0;							// we don't need it.
 
@@ -553,18 +672,27 @@ bool s_WML_Listener::populateStrux(PL_StruxDocHandle sdh,
 
 	case PTX_EndEndnote:
 	case PTX_EndFootnote:
-	case PTX_EndTOC:
 	case PTX_SectionEndnote:
 	case PTX_SectionFootnote:
-	case PTX_SectionTOC:
 	{
 		return true;
 	}
 
+	case PTX_SectionTOC:
+	{
+		_emitTOC(pcr->getIndexAP());
+		return true;
+	}
+
+	case PTX_EndTOC:
+	  {
+	    return true;
+	  }
+
 	case PTX_EndMarginnote:
 	case PTX_SectionMarginnote:
 	default:
-		UT_ASSERT(UT_TODO);
+		UT_ASSERT_HARMLESS(UT_TODO);
 		return true;
 	}
 }
@@ -572,7 +700,7 @@ bool s_WML_Listener::populateStrux(PL_StruxDocHandle sdh,
 bool s_WML_Listener::change(PL_StruxFmtHandle /*sfh*/,
 			       const PX_ChangeRecord * /*pcr*/)
 {
-  UT_ASSERT(0);	    // this function is not used.
+  UT_ASSERT_NOT_REACHED();	    // this function is not used.
   return false;
 }
 
@@ -584,13 +712,13 @@ bool s_WML_Listener::insertStrux(PL_StruxFmtHandle /*sfh*/,
 								 PL_ListenerId /* lid */,
 																 PL_StruxFmtHandle /* sfhNew */))
 {
-  UT_ASSERT(0);	    // this function is not used.
+  UT_ASSERT_NOT_REACHED();	    // this function is not used.
   return false;
 }
 
 bool s_WML_Listener::signal(UT_uint32 /* iSignal */)
 {
-	UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
+	UT_ASSERT_NOT_REACHED();
 	return false;
 }
 
@@ -669,11 +797,11 @@ void s_WML_Listener::_handleEmbedded(PT_AttrPropIndex api)
 	//this should never happen, but it doesn't hurt to make sure
 	if((m_bInTable && (!m_bInRow || !m_bInCell)))
 	{
-		UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
+		UT_ASSERT_HARMLESS(UT_SHOULD_NOT_HAPPEN);
 		return;
 	}
 
-	const XML_Char* szValue = 0;
+	const gchar* szValue = 0;
 	const PP_AttrProp * pAP = NULL;
 	bool bHaveProp = m_pDocument->getAttrProp(api,&pAP);
 
@@ -682,12 +810,12 @@ void s_WML_Listener::_handleEmbedded(PT_AttrPropIndex api)
 	UT_UTF8String buf = "snapshot-png-";
 	buf += szValue;
 
-	char* dataid = UT_strdup(buf.utf8_str());
+	char* dataid = g_strdup(buf.utf8_str());
 	m_utvDataIDs.push_back(dataid);
 	buf += ".png";
 
 	m_pie->write("<img alt=\"AbiWord Chart\" src=\""); //an alt attribute is required in WML
-	m_pie->write(UT_basename(m_pie->getFileName()));
+	m_pie->write(UT_go_basename(m_pie->getFileName()).utf8_str());
 	m_pie->write("_data/");
 	m_pie->write(buf.utf8_str());
 	m_pie->write("\"");
@@ -728,12 +856,12 @@ void s_WML_Listener::_handleField(const PX_ChangeRecord_Object * pcro, PT_AttrPr
 	//this can happen with certain .doc files
 	if((m_bInTable && (!m_bInRow || !m_bInCell)))
 	{
-		UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
+		UT_ASSERT_HARMLESS(UT_SHOULD_NOT_HAPPEN);
 		return;
 	}
 
 	const PP_AttrProp * pAP = NULL;
-	const XML_Char* szValue = NULL;
+	const gchar* szValue = NULL;
 	bool bHaveProp = m_pDocument->getAttrProp(api,&pAP);
 
 	UT_return_if_fail(bHaveProp && pAP && pAP->getAttribute("type", szValue));
@@ -741,7 +869,9 @@ void s_WML_Listener::_handleField(const PX_ChangeRecord_Object * pcro, PT_AttrPr
 	UT_UTF8String buf;
 	fd_Field * field = pcro->getField();
 
-	if(UT_strcmp(szValue, "list_label") != 0)
+	m_pie->populateFields ();
+
+	if(strcmp(szValue, "list_label") != 0)
 	{
 		buf = field->getValue();  //retrieve the field text
 		buf.escapeXML();
@@ -761,12 +891,12 @@ void s_WML_Listener::_handleBookmark(PT_AttrPropIndex api)
 	//this can happen with certain .doc files
 	if((m_bInTable && (!m_bInRow || !m_bInCell)))
 	{
-		UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
+		UT_ASSERT_HARMLESS(UT_SHOULD_NOT_HAPPEN);
 		return;
 	}
 
 	const PP_AttrProp * pAP = NULL;
-	const XML_Char* szValue = NULL;
+	const gchar* szValue = NULL;
 	bool bHaveProp = m_pDocument->getAttrProp(api,&pAP);
 	UT_UTF8String buf;
 
@@ -774,7 +904,7 @@ void s_WML_Listener::_handleBookmark(PT_AttrPropIndex api)
 	{
 		_closeAnchor(); //nested anchors are not allowed
 
-		if(UT_strcmp(szValue, "start") == 0 && (m_bInHyperlink == 0) && pAP->getAttribute("name", szValue))
+		if(strcmp(szValue, "start") == 0 && (m_bInHyperlink == 0) && pAP->getAttribute("name", szValue))
 		{
 			buf = szValue;
 			buf.escapeXML();
@@ -798,12 +928,12 @@ void s_WML_Listener::_handleHyperlink(PT_AttrPropIndex api)
 	//this can happen with certain .doc files
 	if((m_bInTable && (!m_bInRow || !m_bInCell)))
 	{
-		UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
+		UT_ASSERT_HARMLESS(UT_SHOULD_NOT_HAPPEN);
 		return;
 	}
 
 	const PP_AttrProp * pAP = NULL;
-	const XML_Char* szValue = NULL;
+	const gchar* szValue = NULL;
 	bool bHaveProp = m_pDocument->getAttrProp(api,&pAP);
 	UT_UTF8String buf;
 
@@ -836,11 +966,11 @@ void s_WML_Listener::_handleMath(PT_AttrPropIndex api)
 	//this should never happen, but it doesn't hurt to make sure
 	if((m_bInTable && (!m_bInRow || !m_bInCell)))
 	{
-		UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
+		UT_ASSERT_HARMLESS(UT_SHOULD_NOT_HAPPEN);
 		return;
 	}
 
-	const XML_Char* szValue = 0;
+	const gchar* szValue = 0;
 	const PP_AttrProp * pAP = NULL;
 	bool bHaveProp = m_pDocument->getAttrProp(api,&pAP);
 
@@ -849,12 +979,12 @@ void s_WML_Listener::_handleMath(PT_AttrPropIndex api)
 	UT_UTF8String buf = "snapshot-png-";
 	buf += szValue;
 
-	char *dataid = UT_strdup(buf.utf8_str());
+	char *dataid = g_strdup(buf.utf8_str());
 	m_utvDataIDs.push_back(dataid);
 	buf += ".png";
 
 	m_pie->write("<img alt=\"AbiWord Equation\" src=\""); //an alt attribute is required in WML
-	m_pie->write(UT_basename(m_pie->getFileName()));
+	m_pie->write(UT_go_basename(m_pie->getFileName()).utf8_str());
 	m_pie->write("_data/");
 	m_pie->write(buf.utf8_str());
 	m_pie->write("\"");
@@ -877,21 +1007,21 @@ void s_WML_Listener::_handleImage(PT_AttrPropIndex api)
 	//this should never happen, but it doesn't hurt to make sure
 	if((m_bInTable && (!m_bInRow || !m_bInCell)))
 	{
-		UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
+		UT_ASSERT_HARMLESS(UT_SHOULD_NOT_HAPPEN);
 		return;
 	}
 
 	const PP_AttrProp * pAP = NULL;
-	const XML_Char* szValue = NULL;
+	const gchar* szValue = NULL;
 	bool bHaveProp = m_pDocument->getAttrProp(api,&pAP);
 
 	UT_return_if_fail(bHaveProp && pAP && pAP->getAttribute("dataid", szValue));
 
 	UT_UTF8String buf;
-	char * dataid = UT_strdup(static_cast<const char*>(szValue));
+	char * dataid = g_strdup(static_cast<const char*>(szValue));
 	m_utvDataIDs.push_back(dataid);
 
-	char * temp = _stripSuffix(UT_basename(szValue), '_');
+	char * temp = _stripSuffix(UT_go_basename(szValue), '_');
 	char * fstripped = _stripSuffix(temp, '.');
 	UT_UTF8String_sprintf(buf, "%s.png", fstripped);
 
@@ -913,13 +1043,13 @@ void s_WML_Listener::_handleImage(PT_AttrPropIndex api)
 	}
 
 	m_pie->write("\" src=\"");
-	m_pie->write(UT_basename(m_pie->getFileName()));
+	m_pie->write(UT_go_basename(m_pie->getFileName()).utf8_str());
 	m_pie->write("_data/");
 	m_pie->write(buf.utf8_str());
 	m_pie->write("\"");
 
-	const XML_Char * szWidth = 0;
-	const XML_Char * szHeight = 0;
+	const gchar * szWidth = 0;
+	const gchar * szHeight = 0;
 	UT_LocaleTransactor t(LC_NUMERIC, "C");
 
 	if(pAP->getProperty("width", szWidth))
@@ -960,21 +1090,21 @@ void s_WML_Listener::_handlePositionedImage(PT_AttrPropIndex api)
 	//this should never happen, but it doesn't hurt to make sure
 	if((m_bInTable && (!m_bInRow || !m_bInCell)))
 	{
-		UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
+		UT_ASSERT_HARMLESS(UT_SHOULD_NOT_HAPPEN);
 		return;
 	}
 
 	const PP_AttrProp * pAP = NULL;
-	const XML_Char* szValue = NULL;
+	const gchar* szValue = NULL;
 	bool bHaveProp = m_pDocument->getAttrProp(api,&pAP);
 
 	UT_return_if_fail(bHaveProp && pAP && pAP->getAttribute("strux-image-dataid", szValue));
 
 	UT_UTF8String buf;
-	char * dataid = UT_strdup(static_cast<const char*>(szValue));
+	char * dataid = g_strdup(static_cast<const char*>(szValue));
 	m_utvDataIDs.push_back(dataid);
 
-	char * temp = _stripSuffix(UT_basename(szValue), '_');
+	char * temp = _stripSuffix(UT_go_basename(szValue), '_');
 	char * fstripped = _stripSuffix(temp, '.');
 	UT_UTF8String_sprintf(buf, "%s.png", fstripped);
 
@@ -996,13 +1126,13 @@ void s_WML_Listener::_handlePositionedImage(PT_AttrPropIndex api)
 	}
 
 	m_pie->write("\" src=\"");
-	m_pie->write(UT_basename(m_pie->getFileName()));
+	m_pie->write(UT_go_basename(m_pie->getFileName()).utf8_str());
 	m_pie->write("_data/");
 	m_pie->write(buf.utf8_str());
 	m_pie->write("\"");
 
-	const XML_Char * szWidth = 0;
-	const XML_Char * szHeight = 0;
+	const gchar * szWidth = 0;
+	const gchar * szHeight = 0;
 	UT_LocaleTransactor t(LC_NUMERIC, "C");
 
 	if(pAP->getProperty("width", szWidth))
@@ -1049,7 +1179,7 @@ void s_WML_Listener::_handleDataItems(void)
 		UT_sint32 loc = -1;
 		for (UT_uint32 i = 0; i < m_utvDataIDs.getItemCount(); i++)
 		{
-			if(UT_strcmp(static_cast<const char*>(m_utvDataIDs[i]), szName) == 0)
+			if(strcmp(static_cast<const char*>(m_utvDataIDs[i]), szName) == 0)
 			{
 				loc = i;
 				break;
@@ -1058,51 +1188,41 @@ void s_WML_Listener::_handleDataItems(void)
 
 		if(loc > -1)
 		{
-			FILE *fp;
-			UT_String fname;
+			UT_UTF8String fname;
 #ifndef LEGIONS
-			UT_String_sprintf(fname, "%s_data", m_pie->getFileName());
+			UT_UTF8String_sprintf(fname, "%s_data", m_pie->getFileName());
 #else
 			fname = "images";
 #endif
-			UT_sint32 result = m_pDocument->getApp()->makeDirectory(fname.c_str(), 0750);
-			if (result < 0)
+			/* UT_sint32 result = */ UT_go_directory_create(fname.utf8_str(), 0750, NULL);
+			if (0 /* result < 0 */)
 			{
 				UT_DEBUGMSG(("WML Export: Failed to create directory\n"));
 				/* we might want to return an error here, 
 				 * but I don't think so. */
 			}
 
-			if (!UT_strcmp(szMimeType, "image/svg+xml"))
-				UT_String_sprintf(fname, "%s/%s_%d.svg", fname.c_str(), szName, loc);
-			if (!UT_strcmp(szMimeType, "application/mathml+xml"))
-				UT_String_sprintf(fname, "%s/%s_%d.mathml", fname.c_str(), szName, loc);
+			if (!strcmp(szMimeType, "image/svg+xml"))
+				UT_UTF8String_sprintf(fname, "%s/%s_%d.svg", fname.utf8_str(), szName, loc);
+			if (!strcmp(szMimeType, "application/mathml+xml"))
+				UT_UTF8String_sprintf(fname, "%s/%s_%d.mathml", fname.utf8_str(), szName, loc);
 			else // PNG Image
 			{
-				char * temp = _stripSuffix(UT_basename(szName), '_');
+				char * temp = _stripSuffix(UT_go_basename(szName), '_');
 				char * fstripped = _stripSuffix(temp, '.');
 				FREEP(temp);
-				UT_String_sprintf(fname, "%s/%s.png", fname.c_str(), fstripped);
+				UT_UTF8String_sprintf(fname, "%s/%s.png", fname.utf8_str(), fstripped);
 				FREEP(fstripped);
 			}
 
-			if (!UT_isRegularFile(fname.c_str()))
-			{
-			    fp = fopen (fname.c_str(), "wb+");
-
-			    if(!fp)
-				    continue;
-
-			    int cnt = 0, len = pByteBuf->getLength();
-
-			    while (cnt < len)
-			    {
-				    cnt += fwrite (pByteBuf->getPointer(cnt),
-						   sizeof(UT_Byte), len-cnt, fp);
-			    }
-
-			    fclose(fp);
-			}
+			GsfOutput *fp = UT_go_file_create (fname.utf8_str(), NULL);
+			
+			if(!fp)
+			  continue;
+			
+			gsf_output_write(fp, pByteBuf->getLength(), (const guint8 *)pByteBuf->getPointer(0));
+			gsf_output_close(fp);
+			g_object_unref(G_OBJECT(fp));
 		}
 	}
 }
@@ -1122,16 +1242,16 @@ void s_WML_Listener::_openSpan(PT_AttrPropIndex api)
 
 	if (bHaveProp && pAP)
 	{
-		const XML_Char * szValue = NULL;
+		const gchar * szValue = NULL;
 
 		if ((pAP->getProperty("font-weight", szValue))
-			&& !UT_strcmp(szValue, "bold"))
+			&& !strcmp(szValue, "bold"))
 		{
 			m_pie->write("<b>");
 		}
 		
 		if ((pAP->getProperty("font-style", szValue))
-			&& !UT_strcmp(szValue, "italic"))
+			&& !strcmp(szValue, "italic"))
 		{
 			m_pie->write("<i>");
 		}
@@ -1147,11 +1267,11 @@ void s_WML_Listener::_openSpan(PT_AttrPropIndex api)
 		// subscript = small
 		if (pAP->getProperty("text-position", szValue))
 		{
-			if (!UT_strcmp("superscript", szValue))
+			if (!strcmp("superscript", szValue))
 			{
 				m_pie->write("<big>");
 			}
-			else if (!UT_strcmp("subscript", szValue))
+			else if (!strcmp("subscript", szValue))
 			{
 				m_pie->write("<small>");
 			}
@@ -1168,7 +1288,12 @@ void s_WML_Listener::_outputData(const UT_UCSChar * data, UT_uint32 length)
 	{
 		return;
 	}
-	
+
+	_outputDataUnchecked(data, length);
+}
+
+void s_WML_Listener::_outputDataUnchecked(const UT_UCSChar * data, UT_uint32 length)	
+{
 	// TODO deal with unicode.
 	// TODO for now, just squish it into ascii.
 	
@@ -1249,15 +1374,15 @@ void s_WML_Listener::_closeSpan(void)
 	
 	if (pAP)
 	{
-		const XML_Char * szValue = NULL;
+		const gchar * szValue = NULL;
 		
 		if (pAP->getProperty("text-position", szValue))
 		{
-			if (!UT_strcmp("superscript", szValue))
+			if (!strcmp("superscript", szValue))
 			{
 				m_pie->write("</big>");
 			}
-			else if (!UT_strcmp("subscript", szValue))
+			else if (!strcmp("subscript", szValue))
 			{
 				m_pie->write("</small>");
 			}
@@ -1270,13 +1395,13 @@ void s_WML_Listener::_closeSpan(void)
 		}
 
 		if ((pAP->getProperty("font-style", szValue))
-			&& !UT_strcmp(szValue, "italic"))
+			&& !strcmp(szValue, "italic"))
 		{
 		  m_pie->write("</i>");
 		}
 		
 		if ((pAP->getProperty("font-weight", szValue))
-			&& !UT_strcmp(szValue, "bold"))
+			&& !strcmp(szValue, "bold"))
 		{
 		  m_pie->write("</b>");
 		}
