@@ -68,7 +68,7 @@ HDC GR_Win32Graphics::m_defPrintHDC = NULL;
 /*****************************************************************/
 
 // A small helper class
-class private_FontReverter
+class ABI_EXPORT private_FontReverter
 {
 public:
 	private_FontReverter(GR_Win32Graphics& gr, GR_Font* pOldFont)
@@ -233,7 +233,7 @@ GR_Win32Graphics::~GR_Win32Graphics()
 	for (int n = 0; n<m_nArPenPos; n++, pArPens++)
 		DeleteObject(pArPens->hPen);	
 
-	delete m_pArPens;
+	DELETEPV(m_pArPens);
 
 	DELETEP(m_pFontGUI);
 	
