@@ -517,8 +517,8 @@ static const char* search_rmap_with_opt_suffix(const _rmap* m,const char* key,co
 
 struct _map
 {
-	char* key;
-	char* value;
+	const char* key;
+	const char* value;
 };
 static const char* search_map(const _map* m,const char* key,bool* is_default = NULL)
 {
@@ -1405,7 +1405,7 @@ const char* XAP_EncodingManager::charsetFromCodepage(int lid) const
     return is_default ? cpname : ret;
 }
 
-const char* XAP_EncodingManager::CodepageFromCharset(char *charset) const
+const char* XAP_EncodingManager::CodepageFromCharset(const char *charset) const
 {
     bool is_default;
     const char* ret = search_map(charset_name_to_MSCodepagename_map,charset,&is_default);
@@ -1564,7 +1564,7 @@ static enum EUniCat categoriseUniChar(UT_UCS4Char c) {
 	// use linear search for the bottom (western part of the table, and bsearch for the
 	// rest
 
-	SCatRange * pUC = (SCatRange*)bsearch(&c, UniCharCats, NrElements(UniCharCats), sizeof(SCatRange),
+	SCatRange * pUC = (SCatRange*)bsearch(&c, UniCharCats, G_N_ELEMENTS(UniCharCats), sizeof(SCatRange),
 											  s_compare_unichar_cats);
 
 	if(pUC)
