@@ -25,6 +25,10 @@
 
 #define ABIWORD_INTERNAL
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -40,8 +44,8 @@
 #include "ut_misc.h"
 #include "ut_locale.h"
 #ifdef ABI_OPT_PERL
-#include "ut_PerlBindings.h"
-#include "ut_Script.h"
+  #include "ut_PerlBindings.h"
+  #include "ut_Script.h"
 #endif
 #include "ut_unixDirent.h"
 #include "ut_sleep.h"
@@ -52,8 +56,8 @@
 #include "ap_UnixFrame.h"
 #include "ap_UnixApp.h"
 
-#ifndef WITHOUT_SPELL
-#include "spell_manager.h"
+#ifdef ENABLE_SPELL
+  #include "spell_manager.h"
 #endif
 
 #include "ap_Strings.h"
@@ -111,9 +115,10 @@
 #include "ap_Preview_Abi.h"
 #include "xap_UnixDialogHelper.h"
 #include "gr_UnixPangoGraphics.h"
+#include "gr_UnixPangoPixmapGraphics.h"
 
 #ifdef ENABLE_BINRELOC
-#include "prefix.h"
+  #include "prefix.h"
 #endif // ENABLE_BINRELOC
 
 
@@ -127,24 +132,16 @@
 #include "ie_impGraphic.h"
 #include "ut_math.h"
 
-#ifndef WITHOUT_PRINTING
-#include <libart_lgpl/art_affine.h>
-#include <libgnomeprint/gnome-print.h>
+#ifdef ENABLE_PRINT
+  #include <libart_lgpl/art_affine.h>
+  #include <libgnomeprint/gnome-print.h>
 #endif
 
-#ifdef HAVE_GNOMEUI
-#include <libgnome/libgnome.h>
-#include <libgnomeui/libgnomeui.h>
+#ifdef WITH_GNOMEUI
+  #include <libgnome/libgnome.h>
+  #include <libgnomeui/libgnomeui.h>
 #endif
 
-#ifdef HAVE_BONOBO
-#include <libbonoboui.h>
-#include <bonobo/bonobo-macros.h>
-#include <bonobo/bonobo-object.h>
-#include "ap_EditMethods.h"
-#include "abiwidget.h"
-static int mainBonobo(int argc, const char ** argv);
-#endif
 #ifdef LOGFILE
 static FILE * logfile;
 extern FILE * getlogfile(void)
@@ -172,11 +169,9 @@ AP_UnixApp::AP_UnixApp(XAP_Args * pArgs, const char * szAppName)
 	  m_cacheSelectionView(0),
 	  m_pFrameSelection(0)
 {
-#ifndef HAVE_BONOBO
     // hack to link abi_widget - thanks fjf
 	if(this == 0)
 		/*GtkWidget * pUn =*/ abi_widget_new_with_file("fred.abw");
-#endif
 }
 
 /*!
@@ -672,9 +667,6 @@ void AP_UnixApp::pasteFromClipboard(PD_DocumentRange * pDocRange, bool bUseClipb
 
     if (AP_UnixClipboard::isRichTextTag(szFormatFound))
     {
-		// iLen includes the terminating '\0'
-		UT_ASSERT_HARMLESS(iLen == strnlen((const char *) pData, iLen) + 1);
-
 		IE_Imp_RTF * pImpRTF = new IE_Imp_RTF(pDocRange->m_pDoc);
 		bSuccess = pImpRTF->pasteFromBuffer(pDocRange,pData,iLen);
 		DELETEP(pImpRTF);
@@ -754,8 +746,6 @@ void AP_UnixApp::pasteFromClipboard(PD_DocumentRange * pDocRange, bool bUseClipb
       }
     else // ( AP_UnixClipboard::isTextTag(szFormatFound) )
     {
-		UT_ASSERT_HARMLESS(iLen == strnlen((const char *) pData, iLen));
-		
 		IE_Imp_Text * pImpText = new IE_Imp_Text(pDocRange->m_pDoc,"UTF-8");
 		bSuccess = pImpText->pasteFromBuffer(pDocRange,pData,iLen);
 		DELETEP(pImpText);
@@ -766,7 +756,6 @@ void AP_UnixApp::pasteFromClipboard(PD_DocumentRange * pDocRange, bool bUseClipb
 	// we failed to paste *anything.* try plaintext as a last-ditch effort
 	if(!bSuccess && m_pClipboard->getTextData(tFrom,reinterpret_cast<const void **>(&pData),&iLen, &szFormatFound)) {
 		UT_DEBUGMSG(("DOM: pasting text as an absolute fallback (bug 7666)\n"));		
-		UT_ASSERT_HARMLESS(iLen == strnlen((const char *) pData, iLen));
 
 		IE_Imp_Text * pImpText = new IE_Imp_Text(pDocRange->m_pDoc,"UTF-8");
 		bSuccess = pImpText->pasteFromBuffer(pDocRange,pData,iLen);
@@ -1145,43 +1134,12 @@ bool AP_UnixApp::getCurrentSelection(const char** formatList,
 
 bool AP_UnixApp:: makePngPreview(const char * pszInFile, const char * pszPNGFile, UT_sint32 iWidth, UT_sint32 iHeight)
 {
-	//
-	// Create a private visual to draw with
-	//
-	GdkVisual * vis = gdk_visual_get_system (); // don't delete this!
-	//
-	// Create a private color map to draw with.
-	//
-	GdkColormap*  visColorMap = gdk_colormap_new(vis,true);
-	//
-	// Create a private set of attributes for our GdkWindow
-	// 
-	GdkWindowAttr attributes;
-	attributes.title = NULL;
-	attributes.event_mask = 0;
-	attributes.x = 0;
-	attributes.y = 0;
-	attributes.width = iWidth;
-	attributes.height = iHeight;
-	attributes.wclass = 	GDK_INPUT_OUTPUT ;
-	attributes.visual = vis;
-	attributes.colormap = visColorMap;
-	attributes.window_type = GDK_WINDOW_CHILD;
-	attributes.cursor = NULL;
-	attributes.wmclass_name = NULL;
-	attributes.wmclass_class = NULL;
-	attributes.override_redirect = true;
+	GdkPixmap*  pPixmap = gdk_pixmap_new(NULL,iWidth,iHeight,24);
 
-	gint attributes_mask = GDK_WA_X | GDK_WA_Y | GDK_WA_COLORMAP | GDK_WA_VISUAL;
-	GdkWindow*  pWindow = gdk_window_new (NULL, &attributes,
-                                             attributes_mask);
-#if 0
-	GdkPixmap * pMap =  gdk_pixmap_new(pWindow,iWidth,iHeight,-1); 	
-#endif
-	GR_UnixAllocInfo ai(pWindow);
-	GR_UnixPangoGraphics * pG = (GR_UnixPangoGraphics*) XAP_App::getApp()->newGraphics(ai);
+	GR_UnixPixmapAllocInfo ai(pPixmap);
+
+	GR_UnixPangoPixmapGraphics * pG = (GR_UnixPangoPixmapGraphics*) GR_UnixPangoPixmapGraphics::graphicsAllocator(ai);
 
-	pG->createCaret();
 	UT_Error error = UT_OK;
 	PD_Document * pNewDoc = new PD_Document(this);
 	error = pNewDoc->readFromFile(pszInFile,IEFT_Unknown, NULL);
@@ -1194,8 +1152,9 @@ bool AP_UnixApp:: makePngPreview(const char * pszInFile, const char * pszPNGFile
 	dg_DrawArgs da;
 	memset(&da, 0, sizeof(da));
 	da.pG = pG;
-	pPrevAbi->getView()->draw(0, &da);
 	GR_Painter * pPaint = new GR_Painter(pG);
+	pPaint->clearArea(0,0,pG->tlu(iWidth),pG->tlu(iHeight));
+	pPrevAbi->getView()->draw(0, &da);
 	UT_Rect r;
 	r.left = 0;
 	r.top = 0;
@@ -1206,194 +1165,12 @@ bool AP_UnixApp:: makePngPreview(const char * pszInFile, const char * pszPNGFile
 	static_cast<GR_UnixImage *>(pImage)->saveToPNG( pszPNGFile);
 	DELETEP(pImage);
 	DELETEP(pG);
-	delete pWindow;
-#if 0
-	delete pMap;
-#endif
-	delete visColorMap;
 	DELETEP(pPrevAbi); // This deletes pNewDoc
 	return true;
 }
 
 /*****************************************************************/
 /*****************************************************************/
-#if !defined(EMBEDDED_TARGET) || defined(EMBEDDED_SPLASH)
-static GtkWidget * wSplash = NULL;
-static GR_Image * pSplashImage = NULL;
-static GR_UnixPangoGraphics * pUnixGraphics = NULL;
-static bool firstExpose = FALSE;
-static UT_uint32 splashTimeoutValue = 0;
-
-static guint death_timeout_handler = 0;
-
-/*!
-  Hide the splash image.
-  \todo Shouldn't this return a bool?
-  \todo Should we check the return values from the GTK calls?
-  \return Always returns TRUE (1)
-*/
-static gint s_hideSplash(gpointer /*data*/)
-{
-    if (wSplash)
-    {
-		g_source_remove(death_timeout_handler);
-		gtk_widget_destroy(wSplash);
-		wSplash = NULL;
-		DELETEP(pUnixGraphics);
-		DELETEP(pSplashImage);
-    }
-    return TRUE;
-}
-
-/*!
-  Calls s_hideSplash().  Used for button clicks before the regular app
-  starts.  This only gets called if the button click is on the splash
-  screen.  
-*/
-static void s_button_event(GtkWidget * /*window*/)
-{
-    s_hideSplash(NULL);
-}
-
-/*!
-  Actually draws the splash screen.  
-  \return Always returns FALSE.
-  \bug When boolean functions return meaningless values, they should
-	return TRUE, not FALSE.  
-*/
-static gint s_drawingarea_expose(GtkWidget * /* widget */,
-								 GdkEventExpose * /* pExposeEvent */)
-{
-    if (pUnixGraphics && pSplashImage)
-    {
-		GR_Painter painter (pUnixGraphics);
-		painter.drawImage(pSplashImage, 0, 0);
-
-		// on the first full paint of the image, start a 2 second timer
-		if (!firstExpose)
-		{
-			firstExpose = true;
-			// kill the window after splashTimeoutValue ms
-			death_timeout_handler = g_timeout_add_full(0,splashTimeoutValue, s_hideSplash, NULL,NULL);
-		}
-    }
-
-    return FALSE;
-}
-
-#ifdef DEBUG // This code is only used by ifdef'ed code in _showSplash
-const char * _getUserPrivateDirectory(void)
-{
-	/* return a pointer to a static buffer */
-	
-	char * szAbiDir = ".AbiSuite";
-	
-	static char buf[PATH_MAX];
-	memset(buf,0,sizeof(buf));
-	
-	char * szHome = getenv("HOME");
-	if (!szHome || !*szHome)
-		szHome = "./";
-	
-	if (strlen(szHome)+strlen(szAbiDir)+2 >= PATH_MAX)
-		return NULL;
-	
-	strcpy(buf,szHome);
-	if (buf[strlen(buf)-1] != '/')
-		strcat(buf,"/");
-	strcat(buf,szAbiDir);
-	return buf;
-}
-#endif
-
-/*! 
-  szFile is optional; a NULL pointer will use the default splash screen.
-  \param delay How long the splash should stay on screen in milliseconds.
-  \return Pointer to the splash image.  
-*/
-GR_Image * AP_UnixApp::_showSplash(UT_uint32 delay)
-{
-    wSplash = NULL;
-    pSplashImage = NULL;
-
-    UT_ByteBuf* pBB = NULL;
-
-    // store value for use by the expose event, which attaches the timer
-    splashTimeoutValue = delay;
-	
-    extern unsigned char g_pngSplash[];		// see ap_wp_Splash.cpp
-    extern unsigned long g_pngSplash_sizeof;	// see ap_wp_Splash.cpp
-	
-    pBB = new UT_ByteBuf();
-    if (
-	(pBB->ins(0, g_pngSplash, g_pngSplash_sizeof))
-	)
-      {
-		// get splash size
-		UT_sint32 iSplashWidth;
-		UT_sint32 iSplashHeight;
-		{
-			bool pngReturnVal = UT_PNG_getDimensions(pBB, iSplashWidth, iSplashHeight);
-			if(!pngReturnVal)
-			{
-				// uh oh
-				UT_DEBUGMSG(("That image caused an error in the PNG library."));
-				return NULL;
-			}
-		}
-		// create a centered window the size of our image
-		wSplash = gtk_window_new(GTK_WINDOW_TOPLEVEL); //GTK_WINDOW_DIALOG
-		gtk_window_set_decorated (GTK_WINDOW(wSplash), FALSE);
-		gtk_window_set_default_size (GTK_WINDOW (wSplash),
-									 iSplashWidth, iSplashHeight);
-		gtk_window_set_resizable(GTK_WINDOW(wSplash), FALSE);
-
-		// create a frame to add depth
-		GtkWidget * frame = gtk_frame_new(NULL);
-		gtk_container_add(GTK_CONTAINER(wSplash), frame);
-		gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_OUT);
-		gtk_widget_show(frame);
-
-		// create a drawing area
-		GtkWidget * da = createDrawingArea ();
-		gtk_widget_set_events(da, GDK_ALL_EVENTS_MASK);
-		gtk_widget_set_size_request(da, iSplashWidth, iSplashHeight);
-		g_signal_connect(G_OBJECT(da), "expose_event",
-						   G_CALLBACK(s_drawingarea_expose), NULL);
-		g_signal_connect(G_OBJECT(da), "button_press_event",
-						   G_CALLBACK(s_button_event), NULL);
-		gtk_container_add(GTK_CONTAINER(frame), da);
-		gtk_widget_show(da);
-
-		// now bring the window up front & center
-		gtk_window_set_position(GTK_WINDOW(wSplash), WIN_POS);
-
-		// create the window so we can attach a GC to it
-		gtk_widget_show(wSplash);
-		
-		// create image context
-		GR_UnixAllocInfo ai(da->window);
-		pUnixGraphics =
-			(GR_UnixPangoGraphics*) XAP_App::getApp()->newGraphics(ai);
-		
-		pSplashImage =
-			pUnixGraphics->createNewImage("splash", pBB,
-										  pUnixGraphics->tlu(iSplashWidth), 
-										  pUnixGraphics->tlu(iSplashHeight));
-
-		// another for luck (to bring it up forward and paint)
-		gtk_widget_show(wSplash);
-
-		// trigger an expose event to get us started
-		s_drawingarea_expose(da, NULL);
-    }
-
-    DELETEP(pBB);
-
-    return pSplashImage;
-}
-#endif
-
 GR_Graphics * AP_UnixApp::newDefaultScreenGraphics() const
 {
 	XAP_Frame * pFrame = findValidFrame();
@@ -1449,7 +1226,7 @@ int AP_UnixApp::main(const char * szAppName, int argc, const char ** argv)
 #endif
 
 		if (have_display > 0) {
-#ifndef HAVE_GNOMEUI
+#ifndef WITH_GNOMEUI
 			gtk_init (&XArgs.m_argc,const_cast<char ***>(&XArgs.m_argv));
 			Args.parsePoptOpts();
 #else
@@ -1475,15 +1252,6 @@ int AP_UnixApp::main(const char * szAppName, int argc, const char ** argv)
 			fprintf(logfile,"g_object_get completed \n");
 #endif
 
-#ifdef HAVE_BONOBO
-#ifdef LOGFILE
-			fprintf(logfile,"About to init bonobo \n");
-#endif	  
-			bonobo_init (&XArgs.m_argc, const_cast<char **>(XArgs.m_argv));
-#ifdef LOGFILE
-			fprintf(logfile,"bonobo initialized \n");
-#endif
-#endif
 			// GNOME handles 'parsePoptOpts'.  Isn't it grand?
 #endif
 		}
@@ -1532,47 +1300,6 @@ int AP_UnixApp::main(const char * szAppName, int argc, const char ** argv)
 
 		if (have_display) {
 
-#ifdef HAVE_BONOBO
-			//
-			// Check to see if we've been activated as a control by OAF
-			//
-			bool bControlFactory = false;
-			for (UT_sint32 k = 1; k < XArgs.m_argc; k++)
-			{
-				if (*XArgs.m_argv[k] == '-')
-					if (strstr(XArgs.m_argv[k],"GNOME_AbiWord_ControlFactory") != 0)
-					{
-						bControlFactory = true;
-						break;
-					}
-			}
-			if(bControlFactory)
-			{
-				int rtn = mainBonobo(XArgs.m_argc, XArgs.m_argv);
-#ifdef LOGFILE
-				fprintf(logfile,"mainBonobo Finished \n");
-				fclose(logfile);
-#endif
-				pMyUnixApp->shutdown();
-				delete pMyUnixApp;
-				return rtn;
-			}
-#endif
-
-#if !defined(EMBEDDED_TARGET) || defined(EMBEDDED_SPLASH)
-			// do we show the splash?
-			bool bShowSplash = Args.getShowSplash();
-		
-			const XAP_Prefs * pPrefs = pMyUnixApp->getPrefs();
-			UT_ASSERT(pPrefs);
-			bool bSplashPref = true;
-			if (pPrefs && 
-				pPrefs->getPrefsValueBool (AP_PREF_KEY_ShowSplash, &bSplashPref))
-				bShowSplash = bShowSplash && bSplashPref;
-		
-			if (bShowSplash)
-				_showSplash(1500);
-#endif
 			// Step 3: Create windows as appropriate.
 			// if some args are botched, it returns false and we should
 			// continue out the door.
@@ -1628,7 +1355,7 @@ void AP_UnixApp::errorMsgBadFile(XAP_Frame * pFrame, const char * file,
  */
 void AP_UnixApp::initPopt (AP_Args * Args)
 {
-#ifdef HAVE_GNOMEUI
+#ifdef WITH_GNOMEUI
 	UT_sint32 v = -1, i;
 
 	// stop at --version.
@@ -1697,7 +1424,7 @@ bool AP_UnixApp::doWindowlessArgs(const AP_Args *Args, bool & bSuccess)
  	AP_UnixApp * pMyUnixApp = static_cast<AP_UnixApp*>(Args->getApp());
 	if (Args->m_sPrintTo) 
 	{
-#ifndef WITHOUT_PRINTING
+#ifdef ENABLE_PRINT
 		if ((Args->m_sFile = poptGetArg (Args->poptcon)) != NULL)
 	    {
 			AP_Convert conv ;
@@ -1756,7 +1483,7 @@ bool AP_UnixApp::doWindowlessArgs(const AP_Args *Args, bool & bSuccess)
 	if (Args->m_iToThumb > 0) 
 	{
 
-#ifndef WITHOUT_PRINTING
+#ifdef ENABLE_PRINT
 
 		if ((Args->m_sFile = poptGetArg (Args->poptcon)) != NULL)
 	    {
@@ -1910,7 +1637,7 @@ static gint s_signal_count = 0;
 /*!
   This function actually handles signals.  The most commonly recieved
   one is SIGSEGV, the segfault signal.  We want to clean up, save the
-  user's files to backup locations (currently <filename>.SEGFAULTED) and then
+  user's files to backup locations (currently <filename>.SAVED) and then
   call abort, so we still get a core dump that we can debug.
   \param sig_num the integer representing which signal we recieved
 */
@@ -1949,7 +1676,7 @@ void AP_UnixApp::catchSignals(int sig_num)
     for(;i<m_vecFrames.getItemCount();i++)
     {
 		AP_UnixFrame * curFrame = const_cast<AP_UnixFrame*>(static_cast<const AP_UnixFrame*>(m_vecFrames[i]));
-		UT_ASSERT(curFrame);
+		UT_continue_if_fail(curFrame);
 		if (NULL == curFrame->getFilename())
 		  curFrame->backup(".abw.SAVED",abiType);
 		else
@@ -1961,766 +1688,3 @@ void AP_UnixApp::catchSignals(int sig_num)
     // Abort and dump core
     abort();
 }
-
-#ifdef HAVE_BONOBO
-
-#include <gsf-gnome/gsf-input-bonobo.h>
-#include <gsf-gnome/gsf-output-bonobo.h>
-
-//-------------------------------------------------------------------
-// Bonobo Control factory stuff
-//-------------------------------------------------------------------
-
-static BonoboControl * AbiWidget_control_new (AbiWidget * abi);
-
-
-/* 
- * get a value from abiwidget
- */ 
-static void get_prop (BonoboPropertyBag 	*bag,
-	  BonoboArg 		*arg,
-	  guint 		 arg_id,
-	  CORBA_Environment 	*ev,
-	  gpointer 		 user_data)
-{
-	GObject 	*abi;
-	
-	g_return_if_fail (IS_ABI_WIDGET(user_data));
-		
-	/*
-	 * get data from our AbiWidget
-	 */
-//
-// first create fresh GValue
-//
-	GValue gVal = {0, };
-	GParamSpec ParamSpec;
-//
-// Now extract the requested GValue from abiwidget using arg_id
-//
-	abi = G_OBJECT(user_data); 
-	abi_widget_get_property(abi,arg_id,&gVal,&ParamSpec);
-//
-// Now copy it back to the bonobo argument.
-//
-	bonobo_arg_from_gvalue (arg, &gVal);
-//
-// Free up allocated memory
-//
-	if (G_VALUE_TYPE(&gVal) == G_TYPE_STRING && g_value_get_string(&gVal))
-	{
-		g_free (const_cast<void*>(static_cast<const void*>(g_value_get_string(&gVal))));
-	}
-}
-
-/*
- * Tell abiwidget to do something.
- */
-static void set_prop (BonoboPropertyBag 	*bag,
-					  const BonoboArg 	*arg,
-					  guint 		 arg_id,
-					  CORBA_Environment 	*ev,
-					  gpointer 		 user_data)
-{
-	GObject 	*abi;
-	
-	g_return_if_fail (IS_ABI_WIDGET(user_data));
-#ifdef LOGFILE
-	fprintf(logfile,"UnixApp::set_prop id %d \n",arg_id);
-#endif
-	abi = G_OBJECT(user_data); 
-//
-// define a fresh GValue and fill it from bonobo_arg
-//
-	GValue gVal = {0, };
-	GParamSpec ParamSpec;
-	bonobo_arg_to_gvalue(&gVal,arg);
-//
-// Now send it to abiwidget via send prop
-//
-	abi_widget_set_property(abi,arg_id,&gVal,&ParamSpec);
-//
-// Free up allocated memory
-//
-//
-// Free up allocated memory
-//
-	if (G_VALUE_TYPE(&gVal) == G_TYPE_STRING && g_value_get_string(&gVal))
-	{
-		g_free (const_cast<void*>(static_cast<const void*>(g_value_get_string(&gVal))));
-	}
-}
-
-
-#if 0
-/****************************************************************
- Whole bunch of bonobo UI verb implementations to enable useful things
- to be done from a merged menuBar.
-**********************************************************************/
-static void
-verb_Undo_cb (BonoboUIComponent *uic, gpointer data, const char *name)
-{
-
-	g_return_if_fail (IS_ABI_WIDGET(data));
-
-	AbiWidget * abi = ABI_WIDGET (G_OBJECT(data));
-	GObject * object = G_OBJECT(abi);
-	AbiWidgetClass * abi_klazz = ABI_WIDGET_CLASS (G_OBJECT_GET_CLASS(object));
-	abi_klazz->undo(abi);
-}
-
-static void
-verb_generic_cb (BonoboUIComponent *uic, gpointer data, const char *name)
-{
-
-	g_return_if_fail (IS_ABI_WIDGET(data));
-
-	AbiWidget * abi = ABI_WIDGET (G_OBJECT(data));
-	GObject * object = G_OBJECT(abi);
-	AbiWidgetClass * abi_klazz = ABI_WIDGET_CLASS (G_OBJECT_GET_CLASS(object));
-	if(strcmp(name,"redo") == 0)
-	{
-		abi_klazz->redo(abi);
-	}
-	else if(strcmp(name,"copy") == 0)
-	{
-		abi_klazz->copy(abi);
-	}
-	else if(strcmp(name,"cut") == 0)
-	{
-		abi_klazz->cut(abi);
-	}
-	else if(strcmp(name,"paste") == 0)
-	{
-		abi_klazz->paste(abi);
-	}
-	else if(strcmp(name,"print") == 0)
-	{
-		abi_widget_invoke(abi,"print");
-	}
-	else if(strcmp(name,"printPreview") == 0)
-	{
-		abi_widget_invoke(abi,"printPreview");
-	}
-	else if(strcmp(name,"fileSave") == 0)
-	{
-		abi_widget_invoke(abi,"fileSave");
-	}
-	else if(strcmp(name,"fileSaveAs") == 0)
-	{
-		abi_widget_invoke(abi,"fileSaveAs");
-	}
-}
-
-static BonoboUIVerb abi_nautilus_verbs[] = {
-	BONOBO_UI_VERB ("fileSave",            verb_generic_cb),
-	BONOBO_UI_VERB ("fileSaveAs",          verb_generic_cb),
-	BONOBO_UI_VERB ("print",               verb_generic_cb),
-	BONOBO_UI_VERB ("printPreview",        verb_generic_cb),
-	BONOBO_UI_VERB ("undo",                verb_Undo_cb),
-	BONOBO_UI_VERB ("redo",                verb_generic_cb),
-	BONOBO_UI_VERB ("copy",                verb_generic_cb),
-	BONOBO_UI_VERB ("cut",                 verb_generic_cb),
-	BONOBO_UI_VERB ("paste",                 verb_generic_cb),
-	BONOBO_UI_VERB_END
-};
-#endif
-
-/*!
- * Do this after
- */
-static void
-abi_nautilus_view_create_ui (AbiWidget *abi)
-{
-#if 0
-	g_return_if_fail (IS_ABI_WIDGET(abi));
-	BonoboUIComponent * uic = abi_widget_get_Bonobo_uic(abi);
-
-	/* Connect the UI component to the control frame's UI container. */
-
-// see nautilus/libnautilus/nautilus-view.c:994
-//  
-//	ui_container = bonobo_control_get_remote_ui_container (view->details->control, NULL);
-//	bonobo_ui_component_set_container (ui_component, ui_container, NULL);
-//	bonobo_object_release_unref (ui_container, NULL);
-
-	/* Set up the UI from XML file. 
-	   Have to find a way to get this from a global
-	installation
-	*/
-	AP_UnixApp * pApp = static_cast<AP_UnixApp *>(XAP_App::getApp());
-	const char * szDir =   pApp->getUserPrivateDirectory();
-	bonobo_ui_util_set_ui (uic, "/home/msevior",
-			       "abi-nautilus-view-file.xml", "AbiWordNautilusView", NULL);
-	bonobo_ui_component_add_verb_list_with_data (uic, abi_nautilus_verbs,
-						     abi);
-#else
-	UT_DEBUGMSG(("DOM: abi_nautilus_view_create_ui not implemented\n"));
-#endif
-}
-
-static void
-control_set_ui_container (AbiWidget *abi,
-			  Bonobo_UIContainer ui_container)
-{
-	g_return_if_fail (IS_ABI_WIDGET(abi));
-	g_return_if_fail (ui_container != CORBA_OBJECT_NIL);
-
-	bonobo_ui_component_set_container (abi_widget_get_Bonobo_uic(abi), ui_container, NULL);
-
-	abi_nautilus_view_create_ui (abi);
-}
-
-static void
-control_unset_ui_container (AbiWidget * abi)
-{
-	g_return_if_fail (IS_ABI_WIDGET(abi));
-
-	bonobo_ui_component_unset_container (abi_widget_get_Bonobo_uic(abi), NULL);
-}
-
-static void abi_control_activate_cb(BonoboControl *object, gboolean state, gpointer data)
-{
-	AbiWidget * abi;
-	g_return_if_fail (IS_ABI_WIDGET(G_OBJECT(data)));
-	abi = ABI_WIDGET(G_OBJECT(data));
-	if(state)
-	{
-		Bonobo_UIContainer ui_container;
-		ui_container = bonobo_control_get_remote_ui_container (object, NULL);
-		if (ui_container != CORBA_OBJECT_NIL) 
-		{
-			control_set_ui_container (abi, ui_container);
-			bonobo_object_release_unref (ui_container, NULL);
-		}
-
-	} 
-	else
-	{
-		control_unset_ui_container (abi);
-	}
-}
-
-
-
-/*****************************************************************/
-/* Implements the Bonobo/Persist:1.0, Bonobo/PersistStream:1.0,
-   Bonobo/PersistFile:1.0 Interfaces */
-/*****************************************************************/
-
-#if 0
-
-// (working) code patiently waiting for Bonobo printing to rise from the grave
-
-static void
-print_document (GnomePrintContext *ctx,
-				double                     inWidth,
-				double                     inHeight,
-				const Bonobo_PrintScissor *opt_scissor,
-				gpointer                   user_data)
-{
-	// assert pre-conditions
-	g_return_if_fail (user_data != NULL);
-	g_return_if_fail (IS_ABI_WIDGET (user_data));
-	
-	// get me!
-	AbiWidget * abi = ABI_WIDGET(user_data);
-	
-	// get our frame
-	XAP_Frame * pFrame = abi_widget_get_frame ( abi ) ;
-	UT_return_if_fail(pFrame != NULL);
-	
-	// get our current view so we can get the document being worked on
-	FV_View * pView = static_cast<FV_View*>(pFrame->getCurrentView());
-	UT_return_if_fail(pView!=NULL);
-	
-	// get the current document
-	PD_Document * pDoc = pView->getDocument () ;
-	UT_return_if_fail(pDoc!=NULL);
-	
-	// get the current app
-	XAP_UnixApp * pApp = static_cast<XAP_UnixApp*>(XAP_App::getApp () );
-	
-	// create a graphics drawing class
-	GR_Graphics *pGraphics = new XAP_UnixGnomePrintGraphics (ctx, inWidth, inHeight) ;
-	UT_return_if_fail(pGraphics!=NULL);
-	
-	// layout the document
-	FL_DocLayout * pDocLayout = new FL_DocLayout(pDoc,pGraphics);
-	UT_ASSERT(pDocLayout);
-	
-	// create a new printing view of the document
-	FV_View printView (pFrame->getApp(),pFrame,pDocLayout);
-	pDocLayout->setView (&printView);
-	pDocLayout->fillLayouts();
-	pDocLayout->formatAll();
-	
-	// get the best fit width & height of the printed pages
-	UT_sint32 iWidth  =  pDocLayout->getWidth();
-	UT_sint32 iHeight =  pDocLayout->getHeight();
-	//UT_sint32 iPages  = pDocLayout->countPages();
-	UT_uint32 width   = MIN(iWidth, pGraphics->tluD(inWidth));
-	UT_uint32 height  = MIN(iHeight, pGraphics->tluD(inHeight));
-	
-	// figure out roughly how many pages to print
-	UT_sint32 iPagesToPrint = static_cast<UT_sint32>(height/pGraphics->tluD(pDoc->m_docPageSize.Height(DIM_PT)));
-	if (iPagesToPrint < 1)
-		iPagesToPrint = 1;
-	
-	// actually print - TODO: can make this better if we figure out what page is focussed
-	s_actuallyPrint (pDoc, pGraphics,
-					 &printView, "bonobo_printed_document",
-					 1, false,
-					 width, height,
-					 1, iPagesToPrint ) ;
-	
-	// clean up
-	DELETEP(pGraphics);
-	DELETEP(pDocLayout);
-}
-
-#endif
-
-#define ABI_BUFFER_SIZE 32768
-
-/*
- * Loads a document from a Bonobo_Stream. Code gratitutously stolen 
- * from ggv
- */
-static void
-load_document_from_stream (BonoboPersistStream *ps,
-					 Bonobo_Stream stream,
-					 Bonobo_Persist_ContentType type,
-					 void *data,
-					 CORBA_Environment *ev)
-{
-	AbiWidget *abiwidget;
-#ifdef LOGFILE
-	fprintf(logfile,"Load file from stream \n");
-#endif
-
-	g_return_if_fail (data != NULL);
-	g_return_if_fail (IS_ABI_WIDGET (data));
-	
-	abiwidget = static_cast<AbiWidget *>(data);
-#ifdef LOGFILE
-	fprintf(logfile,"At entry Load file from stream refcount %d \n",G_OBJECT(abiwidget)->ref_count);
-#endif
-	
-	GsfInput * input = gsf_input_bonobo_new (stream, NULL);
-	g_return_if_fail (input != NULL);
-
-	(void)abi_widget_load_file_from_gsf (abiwidget, input);
-
-	g_object_unref (G_OBJECT (input));
-
-	abi_widget_map_to_screen(abiwidget);
-}
-
-/*
- * Loads a document from a Bonobo_Stream. Code gratitutously stolen 
- * from ggv
- */
-static void
-save_document_to_stream (BonoboPersistStream *ps,
-			 Bonobo_Stream stream,
-			 Bonobo_Persist_ContentType type,
-			 void *data,
-			 CORBA_Environment *ev)
-{
-	AbiWidget *abiwidget;
-
-	g_return_if_fail (data != NULL);
-	g_return_if_fail (IS_ABI_WIDGET (data));
-
-	abiwidget = static_cast<AbiWidget *>(data);
-
-	GsfOutput * output = gsf_output_bonobo_new (stream, NULL);
-
-	(void)abi_widget_save_to_gsf ( abiwidget, output, type );
-
-	gsf_output_close (output);
-	g_object_unref (G_OBJECT (output));
-}
-
-//
-// Implements the get_object interface.
-//
-static Bonobo_Unknown
-abiwidget_get_object(BonoboItemContainer *item_container,
-					 CORBA_char          *item_name,
-					 CORBA_boolean       only_if_exists,
-					 CORBA_Environment   *ev,
-					 AbiWidget *  abi)
-{
-	Bonobo_Unknown corba_object;
-	BonoboObject *object = NULL;
-
-	g_return_val_if_fail(abi != NULL, CORBA_OBJECT_NIL);
-	g_return_val_if_fail(IS_ABI_WIDGET(abi), CORBA_OBJECT_NIL);
-
-	object = BONOBO_OBJECT (AbiWidget_control_new(abi));
-
-	if (object == NULL)
-		return NULL;
-
-	corba_object = bonobo_object_corba_objref (object);
-#ifdef LOGFILE
-	fprintf(logfile," After get_object ref count %d \n",G_OBJECT(object)->ref_count);
-#endif
-
-	return bonobo_object_dup_ref (corba_object, ev);
-}
-
-/*
- * Loads a document from a Bonobo_File. Code gratitutously stolen 
- * from ggv
- */
-static int
-load_document_from_file(BonoboPersistFile *pf, const CORBA_char *filename,
-				   CORBA_Environment *ev, void *data)
-{
-	AbiWidget *abiwidget;
-
-	g_return_val_if_fail (data != NULL,-1);
-	g_return_val_if_fail (IS_ABI_WIDGET (data),-1);
-
-	abiwidget = ABI_WIDGET (data);
-
-	//
-	// Load the file.
-	//
-	g_object_set(G_OBJECT(abiwidget),"AbiWidget--load-file",reinterpret_cast<const gchar *>(filename),NULL);
-	return 0;
-}
-
-static int
-save_document_to_file(BonoboPersistFile *pf, const CORBA_char *filename,
-					  CORBA_Environment *ev, void *data)
-{
-  AbiWidget *abiwidget;
-  
-  g_return_val_if_fail (data != NULL,-1);
-  g_return_val_if_fail (IS_ABI_WIDGET (data),-1);
-  
-  abiwidget = ABI_WIDGET (data);
-
-  abi_widget_save ( abiwidget, filename ) ;
-
-  return 0 ;
-}
-
-//
-// Data content for persist stream
-//
-static Bonobo_Persist_ContentTypeList *
-pstream_get_content_types (BonoboPersistStream *ps, void *closure,
-						   CORBA_Environment *ev)
-{
-	return bonobo_persist_generate_content_types (12, "application/x-abiword", "text/abiword", "application/msword", 
-												  "application/rtf", "text/rtf", "text/richtext", "text/plain", "text/html", "application/xhtml+xml",
-												  "application/x-applix-word", "appplication/vnd.palm", "text/vnd.wap.wml");
-}
-
-/*****************************************************************/
-/* Implements the Bonobo/Zoomable:1.0 Interface */
-/*****************************************************************/
-
-// increment/decrement zoom percentages by this amount
-#define ZOOM_PCTG 10
-
-static float preferred_zoom_levels[] = {
-	1.0 / 4.0, 1.0 / 2.0, 3.0 / 4.0, 1.0, 1.5, 2.0,
-};
-
-static const gint n_zoom_levels = (sizeof (preferred_zoom_levels) / sizeof (float));
-
-static void zoom_level_func(GObject * z, float lvl, gpointer data)
-{
-  g_return_if_fail (data != NULL);
-  g_return_if_fail (IS_ABI_WIDGET(data));
-
-  AbiWidget * abi = ABI_WIDGET(data);
-
-  if ( lvl <= 0.0 )
-    return ;
-
-  XAP_Frame * pFrame = abi_widget_get_frame ( abi ) ;
-  UT_return_if_fail ( pFrame != NULL ) ;
-
-  pFrame->setZoomType (XAP_Frame::z_PERCENT);
-  pFrame->quickZoom (static_cast<UT_uint32>(lvl));
-}
-
-static void zoom_in_func(GObject * z, gpointer data)
-{
-  g_return_if_fail (data != NULL);
-  g_return_if_fail (IS_ABI_WIDGET(data));
-
-  AbiWidget * abi = ABI_WIDGET(data);
-
-  XAP_Frame * pFrame = abi_widget_get_frame ( abi ) ;
-  UT_return_if_fail ( pFrame != NULL ) ;
-
-  UT_sint32 zoom_lvl = pFrame->getZoomPercentage();
-  zoom_lvl += ZOOM_PCTG ;
-
-  pFrame->setZoomType (XAP_Frame::z_PERCENT);
-  pFrame->quickZoom (zoom_lvl);  
-}
-
-static void zoom_out_func(GObject * z, gpointer data)
-{
-  g_return_if_fail (data != NULL);
-  g_return_if_fail (IS_ABI_WIDGET(data));
-
-  AbiWidget * abi = ABI_WIDGET(data);
-
-  XAP_Frame * pFrame = abi_widget_get_frame ( abi ) ;
-  UT_return_if_fail ( pFrame != NULL ) ;
-
-  UT_sint32 zoom_lvl = pFrame->getZoomPercentage();
-  zoom_lvl -= ZOOM_PCTG ;
-
-  if ( zoom_lvl <= 0 )
-    return ;
-
-  pFrame->setZoomType (XAP_Frame::z_PERCENT);
-  pFrame->quickZoom (zoom_lvl);  
-}
-
-static void zoom_to_fit_func(GObject * z, gpointer data)
-{
-  g_return_if_fail (data != NULL);
-  g_return_if_fail (IS_ABI_WIDGET(data));
-
-  AbiWidget * abi = ABI_WIDGET(data);
-
-  XAP_Frame * pFrame = abi_widget_get_frame ( abi ) ;
-  UT_return_if_fail ( pFrame != NULL ) ;
-
-  FV_View * pView = static_cast<FV_View*>(pFrame->getCurrentView());
-  UT_return_if_fail(pView!=NULL);
-
-  UT_uint32 newZoom = pView->calculateZoomPercentForWholePage();
-  pFrame->setZoomType( XAP_Frame::z_WHOLEPAGE );
-  pFrame->quickZoom(newZoom);
-}
-
-static void zoom_to_default_func(GObject * z, gpointer data)
-{
-  g_return_if_fail (data != NULL);
-  g_return_if_fail (IS_ABI_WIDGET(data));
-
-  AbiWidget * abi = ABI_WIDGET(data);
-
-  XAP_Frame * pFrame = abi_widget_get_frame ( abi ) ;
-  UT_return_if_fail ( pFrame != NULL ) ;
-
-  pFrame->setZoomType (XAP_Frame::z_100);
-  pFrame->quickZoom (100);  
-}
-
-/*****************************************************************/
-/* Bonobo Inteface-Adding Code */
-/*****************************************************************/
-
-#define ABIWORD_OAF_IID "OAFIID:GNOME_AbiWord_Control"
-
-//
-// Add extra interfaces to load data into the control
-//
-BonoboObject *
-AbiControl_add_interfaces (AbiWidget *abiwidget,
-						   BonoboObject *to_aggregate)
-{
-	BonoboPersistFile   *file;
-	BonoboPersistStream *stream;
-	BonoboItemContainer *item_container;
-	BonoboZoomable      *zoomable;
-
-	g_return_val_if_fail (IS_ABI_WIDGET(abiwidget), NULL);
-	g_return_val_if_fail (BONOBO_IS_OBJECT (to_aggregate), NULL);
-
-	/* Inteface Bonobo::PropertyBag */
-
-	guint n_pspecs = 0;
-	BonoboPropertyBag * pb = bonobo_property_bag_new (get_prop, set_prop, abiwidget);
-	const GParamSpec ** pspecs = const_cast<const GParamSpec **>(g_object_class_list_properties (G_OBJECT_GET_CLASS (G_OBJECT (abiwidget)), &n_pspecs));
-	bonobo_property_bag_map_params (pb, G_OBJECT (abiwidget), pspecs, n_pspecs);
-	bonobo_control_set_properties (BONOBO_CONTROL(to_aggregate),BONOBO_OBJREF (pb), NULL);
-
-	bonobo_object_add_interface (BONOBO_OBJECT (to_aggregate), BONOBO_OBJECT (pb));
-#ifdef LOGFILE
-	fprintf(logfile,"AbiControl_add_interfaces Property Bag with %d parameters added \n",n_pspecs);
-//	fprintf(logfile," After Property Bag interface ref count %d \n",  bonobo_object_get_ao_ref_count(BONOBO_OBJECT(to_aggregate)));
-#endif
-
-	/* Interface Bonobo::Persist */
-
-	/* Interface Bonobo::PersistStream */
-
-	stream = bonobo_persist_stream_new (load_document_from_stream, 
-										save_document_to_stream, 
-										pstream_get_content_types, 
-										ABIWORD_OAF_IID,
-										abiwidget);
-	if (!stream) {
-		bonobo_object_unref (BONOBO_OBJECT (to_aggregate));
-		return NULL;
-	}
-
-	bonobo_object_add_interface (BONOBO_OBJECT (to_aggregate),
-				     BONOBO_OBJECT (stream));
-#ifdef LOGFILE
-	fprintf(logfile,"AbiControl_add_interfaces stream created \n");
-//	fprintf(logfile," After Stream interface ref count %d \n",  bonobo_object_get_ao_ref_count(BONOBO_OBJECT(to_aggregate)));
-#endif
-
-
-	/* Interface Bonobo::PersistFile */
-
-	file = bonobo_persist_file_new (load_document_from_file,
-									save_document_to_file, 
-									ABIWORD_OAF_IID,
-									abiwidget);
-	if (!file) {
-		bonobo_object_unref (BONOBO_OBJECT (to_aggregate));
-		return NULL;
-	}
-
-	bonobo_object_add_interface (BONOBO_OBJECT (to_aggregate),
-				     BONOBO_OBJECT (file));
-	
-	/* Interface Bonobo/ItemContainer */
-
-	item_container = bonobo_item_container_new ();
-
-	g_signal_connect (G_OBJECT (item_container),
-					  "get_object",
-					  G_CALLBACK (abiwidget_get_object),
-					  abiwidget);
-#ifdef LOGFILE
-//	fprintf(logfile," After get_object signal connect ref count %d \n",  bonobo_object_get_ao_ref_count(BONOBO_OBJECT(to_aggregate)));
-#endif
-	
-	bonobo_object_add_interface (BONOBO_OBJECT (to_aggregate),
-								 BONOBO_OBJECT (item_container));
-
-	/* Interface Bonobo::Zoomable */
-
-	zoomable = bonobo_zoomable_new () ;
-	if ( !zoomable ) {
-	  bonobo_object_unref (BONOBO_OBJECT (to_aggregate));
-	  return NULL;
-	}
-
-	bonobo_zoomable_set_parameters_full
-		(zoomable,
-		 1.0,
-		 0.1,
-		 5.0,
-		 TRUE, TRUE, TRUE,
-		 preferred_zoom_levels,
-		 NULL,
-		 n_zoom_levels);
-
-	g_signal_connect(G_OBJECT(zoomable), "zoom_in",
-			 G_CALLBACK(zoom_in_func), abiwidget);
-	g_signal_connect(G_OBJECT(zoomable), "zoom_out",
-			 G_CALLBACK(zoom_out_func), abiwidget);
-	g_signal_connect(G_OBJECT(zoomable), "zoom_to_fit",
-			 G_CALLBACK(zoom_to_fit_func), abiwidget);
-	g_signal_connect(G_OBJECT(zoomable), "zoom_to_default",
-			 G_CALLBACK(zoom_to_default_func), abiwidget);
-	g_signal_connect(G_OBJECT(zoomable), "set_zoom_level",
-			 G_CALLBACK(zoom_level_func), abiwidget);
-
-	bonobo_object_add_interface (BONOBO_OBJECT (to_aggregate),
-								 BONOBO_OBJECT (zoomable));
-#ifdef LOGFILE
-//	fprintf(logfile," After zoomable connects ref count %d \n",  bonobo_object_get_ao_ref_count(BONOBO_OBJECT(to_aggregate)));
-	fprintf(logfile," After zoomable connects gobject ref count %d \n",G_OBJECT(to_aggregate)->ref_count);
-#endif
-
-/* UI Component */
-	BonoboUIComponent * uic = bonobo_ui_component_new ("AbiWordNautilusView");
-	abi_widget_set_Bonobo_uic(abiwidget,uic);
-
-	g_signal_connect (BONOBO_CONTROL(to_aggregate), "activate", G_CALLBACK (abi_control_activate_cb), abiwidget);
-
-	return to_aggregate;
-}
-
-
-static BonoboControl * AbiWidget_control_new (AbiWidget * abi)
-{
-  // create a BonoboControl from a widget
-#ifdef LOGFILE
-	fprintf(logfile," Just before bonobo_control_new ref count %d \n",G_OBJECT(abi)->ref_count);
-#endif
-  BonoboControl * control = bonobo_control_new (GTK_WIDGET(abi));
-#ifdef LOGFILE
-	fprintf(logfile," Just after bonobo_control_new GOBJECT ref count %d \n",G_OBJECT(control)->ref_count);
-//	fprintf(logfile," Just after bonobo_control_new BOnobo ref count %d \n",  bonobo_object_get_ao_ref_count(BONOBO_OBJECT(control)));
-#endif
-//
-// This fixes the double reference from the bonobo_control_new
-//
-	g_object_unref(G_OBJECT(abi));
-#ifdef LOGFILE
-//	fprintf(logfile," Just after the un_ref count control %d \n", bonobo_object_get_ao_ref_count(BONOBO_OBJECT(control)));
-	fprintf(logfile," Just after the un_ref abi count %d \n",G_OBJECT(abi)->ref_count);
-#endif
-#if 0
-  AbiWidgetClass * abi_klazz = ABI_WIDGET_CLASS (G_OBJECT_GET_CLASS(G_OBJECT(abi)));
-  BonoboObjectClass *bonobo_object_class = (BonoboObjectClass *)abi_klazz;
-  GObjectClass *gobject_class = G_OBJECT_CLASS(abi_klazz);
-#endif
-  AbiControl_add_interfaces (ABI_WIDGET(abi),
-							 BONOBO_OBJECT(control));
-#ifdef LOGFILE
-	fprintf(logfile," After AbiControl_add_interfaces ref count %d \n",G_OBJECT(control)->ref_count);
-#endif
-
-  return control;
-}
-
-/*
- *  produce a brand new bonobo_AbiWord_control
- *  (this is a callback function, registered in 
- *  	'bonobo_generic_factory_new')
- */
-static BonoboObject*
-bonobo_AbiWidget_factory  (BonoboGenericFactory *factory, 
-						   const char           *oaf_iid,
-						   void *closure)
-{
-  /*
-   * create a new AbiWidget instance
-   */  
-  GtkWidget  * abi  = abi_widget_new ();
-#ifdef LOGFILE
-	fprintf(logfile," After new_with_app ref count %d \n",G_OBJECT(abi)->ref_count);
-#endif
-
-	gtk_widget_show (abi);
-#ifdef LOGFILE
-	fprintf(logfile," After gtk_widget_show ref count %d \n",G_OBJECT(abi)->ref_count);
-#endif
-
-  return BONOBO_OBJECT (AbiWidget_control_new (ABI_WIDGET (abi)));
-}
-
-
-
-static int mainBonobo(int argc, const char ** argv)
-{
-	BONOBO_FACTORY_INIT ("abiword-component", "0.1", &argc, const_cast<char **>(argv));
-	XAP_App * pApp = XAP_App::getApp();
-	pApp->setBonoboRunning();
-	return bonobo_generic_factory_main ("OAFIID:GNOME_AbiWord_ControlFactory",
-										bonobo_AbiWidget_factory, NULL);
-}
-
-#endif /* HAVE_BONOBO */
